# import of libraries
import numpy as np

def Fysica_Damping(segments, M_segment, Length_segment, free_space_between_segments, characteristics_small_floater,
                   characteristics_large_floater, z_location_bottom_floater, C_Damping_Pointed_Tip_Reduction, Period_modified, rho, g,
                   Three_Dimensional_Calculation, ExperimentalAddition, draught):

    """"
    # Function:
    # This function makes the damper matrix for the radiation damping according to physical equations

    # Input:
    # segments                          		= Number of segments in the bridge                      	(1)
    # M_segment                         		= Mass of a single segment                              	(1)
    # Length_segment							= Length of a single segment								(1)
    # free_space_between_segments				= Free space between the small floaters between segments	(1)
    # characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
    # characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
    # C_Damping_Pointed_Tip_Reduction			= Radiation damping reductino due to pointed tips
    # Period_modified							= modified wave period to account for current				(3)
    # rho                               		= Density of the fluid                                  	(1)
    # g											= Gravitational acceleration								(1)
    # Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

    # Output:
    # C_added									= Radiation damping matrix									(NxN)
    """

    Wave_Period = Period_modified

    if Three_Dimensional_Calculation == True:
        C_added = np.zeros((6 * segments, 6 * segments))
    else:
        C_added = np.zeros((3, 3))

    # seperate data from arrays which were imported
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Distance between small floaters
    dist_floaters = Length_segment - W_small_floater - free_space_between_segments  # m

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1/2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1/2. * W_large_floater * 0.5 * W_large_floater  # m2

    # x-axis on graph value Vugts calculation
    Wave_w = 2 * np.pi / Wave_Period
    x_small = Wave_w * np.sqrt(W_small_floater / 2 / g)
    x_large = Wave_w * np.sqrt(W_large_floater / 2 / g)

    x = np.array([x_small,x_large])
    y = np.zeros(2)

    # Average length of floaters and value on x-axis on graph Vugts
    B_avg = (L_small_floater * N_small_floater + L_large_floater * N_large_floater) / (N_small_floater + N_large_floater)
    x2 = Wave_w * np.sqrt(B_avg / 2 / g)

    #Curvefit for heave Vugts
    for i in range (2):
        if x[i] < 0.5:
            y[i] = -2.6 * x[i]**2 + 2.1 * x[i]
        elif x[i] > 1.2:
            y[i] = 0.05
        else:
            y[i] = 0.65 - 0.5 * x[i]

    # Reduction because Width/Draught ratio deviates a lot
    y = ( ( ( W_small_floater * N_small_floater + W_large_floater * N_large_floater) /
            (N_small_floater + N_large_floater) ) / draught ) / 2 * y

    # curvefit for sway Vugts
    if x2 < 0.5:
        y2 =0
    elif x2 < 1.75:
        y2 = 0.4674*x2**4 - 2.2721*x2**3 + 3.5827*x2**2 - 1.6391*x2 + 0.1783
    else:
        y2 = 0.5

    # Reduction because of Widht/Draught ratio deviates a lot. + the fact that there are gaps between floaters
    # makes less waves
    GapFactor = 0.5
    y2 = y2 * 8 / (B_avg / draught) * GapFactor


    #Sway
    Byy_unit_width = y2 * rho * B_avg * draught / np.sqrt(B_avg / 2 / g) * C_Damping_Pointed_Tip_Reduction
    Byy = Byy_unit_width * (W_small_floater * N_small_floater + W_large_floater * N_large_floater)


    #Vertical damping per unit length
    Bzz_small_per_length = y[0] * rho / np.sqrt(W_small_floater / 2 / g) * W_small_floater * draught
    Bzz_large_per_length = y[1] * rho / np.sqrt(W_large_floater / 2 / g) * W_large_floater * draught


    #Heave
    Bzz_small = Bzz_small_per_length * (L_small_floater + W_small_floater) * N_small_floater
    Bzz_large = Bzz_large_per_length * (L_large_floater + W_large_floater / 2) * N_large_floater

    #Roll
    Length_small = L_small_floater + 2 * W_small_floater / 2
    Length_large = L_large_floater + W_large_floater / 2

    y_loc_coordinate_small = np.linspace(-Length_small / 2 + Length_small / 28,
                                         Length_small / 2 - Length_small / 28, 14)
    y_loc_coordinate_large = np.linspace(-Length_large / 2 + Length_large / 28,
                                         Length_large / 2 - Length_large / 28,
                                         14)

    L_segment_small = (Length_small / 14)
    L_segment_large = (Length_large / 14)

    Brx_small = 0
    Brx_large = 0

    for i in range (14):
        Brx_small += Bzz_small_per_length * N_small_floater * L_segment_small * y_loc_coordinate_small[i] **2
        Brx_large += Bzz_large_per_length * N_large_floater * L_segment_large * y_loc_coordinate_large[i] **2


    C_added[0,0] = Byy
    C_added[1,1] = Bzz_small + Bzz_large
    C_added[2,2] = Brx_small + Brx_large
    C_added[0,2] = Byy * (-z_location_bottom_floater-draught)
    C_added[2,0] = C_added[0,2]

    return (C_added)



def Fysica_Turbine_Damping(Wave_hlp, rho, g, Turbine_dimensions_HWL, Turbine_char,C_radiation_turbine_sway_reduction):

    """"
    # Function:
    # This function makes the damper matrix for the radiation damping according to physical equations

    # Input:
    # segments                          		= Number of segments in the bridge                      	(1)
    # M_segment                         		= Mass of a single segment                              	(1)
    # Length_segment							= Length of a single segment								(1)
    # free_space_between_segments				= Free space between the small floaters between segments	(1)
    # characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
    # characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
    # C_Damping_Pointed_Tip_Reduction			= Radiation damping reductino due to pointed tips
    # Period_modified							= modified wave period to account for current				(3)
    # rho                               		= Density of the fluid                                  	(1)
    # g											= Gravitational acceleration								(1)
    # Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

    # Output:
    # C_added									= Radiation damping matrix									(NxN)
    """

    Wave_height, Wave_length, Wave_period = Wave_hlp
    C_added = np.zeros((3, 3))
    FFTurbine_H, FFTurbine_W, FFTurbine_L, Turbine_depth, Tur_grid_size, Turbine_Volume, N_turbines, FFTur_evalf_loc = Turbine_dimensions_HWL
    Diameter_turbine_impeller, Induction_factor, C_added_tur_hori, C_added_tur_vert, C_drag_turbine = Turbine_char

    # x-axis on graph value Vugts calculation
    Wave_w = 2 * np.pi / Wave_period
    x_heave = Wave_w * np.sqrt(FFTurbine_L / 2 / g)
    x_sway = Wave_w * np.sqrt(FFTurbine_L / 2 / g)
    x_roll = Wave_w * np.sqrt(FFTurbine_L / 2 / g)

    #Curvefit for heave Vugts
    if x_heave < 0.5:
        y_heave = -2.6 * x_heave**2 + 2.1 * x_heave
    elif x_heave > 1.2:
        y_heave = 0.05
    else:
        y_heave = 0.65 - 0.5 * x_heave
    # Reduction because Width/Draught ratio deviates a lot
    y_heave = ( FFTurbine_L / (FFTurbine_H + Turbine_depth)) / 2 * y_heave

    # curvefit for sway Vugts
    if x_sway < 0.055:
        y_sway =0
    elif x_sway < 0.62:
        y_sway = 4*(x_sway-0.055)**3.2
    elif x_sway < 0.84:
        y_sway = -((x_sway-0.8)/0.3)**2+1
    else:
        y_sway = 2*np.exp(-(x_sway**1.2))+0.09
    
    # curvefit for roll Vugts
    a = 0.07
    if x_roll<0.5:
        y_roll = a*x_roll**2
    elif x_roll>0.5 and x_roll<1:
        y_roll = -a*(x_roll-1)**2 + 0.035
    else:
        y_roll = 0.035
        
    #Sway
    Byy_unit_width = y_sway * rho * FFTurbine_L * (FFTurbine_H + Turbine_depth) / np.sqrt(FFTurbine_L / 2 / g)
    Byy = Byy_unit_width * FFTurbine_W * N_turbines
    #Heave
    Bzz_per_length = y_heave * rho * FFTurbine_L * (FFTurbine_H + Turbine_depth) / np.sqrt(FFTurbine_L / 2 / g)
    Bzz = Bzz_per_length * FFTurbine_W * N_turbines
    #Roll
    Brx_per_length = y_roll * rho  * FFTurbine_L * (FFTurbine_H + Turbine_depth) * FFTurbine_L**2 / np.sqrt(FFTurbine_L / 2 / g)
    Brx = Brx_per_length * FFTurbine_W * N_turbines
    
    C_added[0,0] = Byy * C_radiation_turbine_sway_reduction
    C_added[1,1] = Bzz * (1-C_radiation_turbine_sway_reduction/2)
    C_added[2,2] = Brx * (1-C_radiation_turbine_sway_reduction/2)

    return (C_added)



def Fysica_Floater_Added_Mass(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                characteristics_large_floater, M_segment, z_location_bottom_floater,
                C_Added_Mass_Pointed_Tip_Reduction, C_Added_Mass_Not_Infinite_Factor, C_Marine_Growth_Extra,rho,
                Three_Dimensional_Calculation, Turbine_Force, ExperimentalAddition, draught):
    


    """"
    # Function:
# This function makes the spring / damper matrix for the added mass according to physical equations

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# z_location_turbines						= Z-coordinate of the middle of the turbines				(1)
# Volume_turbine							= Volume displaced water by addition of turbines			(1)
# C_Added_Mass_Pointed_Tip_Reduction		= Factor to add added mass reduction due to pointed tips	(1)
# C_Added_Mass_Not_Infinite_Factor          = Factor to add fact that floaters have not infinite width  (1)
# C_Marine_Growth_Extra						= Added mass coefficient to include marine growth			(1)
# rho                               		= Density of the fluid                                  	(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Turbine_Force								= Option to implement force by current on turbines			(T/F)

# Output:
# M_added									= Added mass matrix											(NxN)
    """

    # seperate data from arrays which were imported
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1/2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1/2. * W_large_floater * 0.5 * W_large_floater  # m2

    # Data taken from DNV report, added drag coefficients for certain b/a ratios
    C_a_square_prisms = [0.68, 0.36, 0.24, 0.19, 0.15, 0.13, 0.11, 0.10, 0.09, 0.08] # for b/a = [1,2,3,4,5,6,7,8,9,10]

    # Added mass coefficient for small floater, horizontal (using interpolation)
    b_a_ratio_small = L_small_floater / np.sqrt(W_small_floater * 2 * draught)
    if b_a_ratio_small > 10:
        C_a_local_small_h = 0.08
    else:
        lower_bound = int(np.floor(b_a_ratio_small))
        higher_bound = int(np.ceil(b_a_ratio_small))
        C_a_local_small_h = abs(higher_bound - b_a_ratio_small) * C_a_square_prisms[lower_bound - 1] + \
                            abs(lower_bound - b_a_ratio_small) * C_a_square_prisms[higher_bound - 1]

    # Added mass coefficient for large floater, horizontal (using interpolation)
    b_a_ratio_large = L_large_floater / np.sqrt(W_large_floater * 2 * draught)
    if b_a_ratio_large > 10:
        C_a_local_large_h = 0.08
    else:
        lower_bound = int(np.floor(b_a_ratio_large))
        higher_bound = int(np.ceil(b_a_ratio_large))
        C_a_local_large_h = abs(higher_bound - b_a_ratio_large) * C_a_square_prisms[lower_bound - 1] + \
                            abs(lower_bound - b_a_ratio_large) * C_a_square_prisms[higher_bound - 1]



    # Data taken from DNV report, added mass coefficients for certain a/b ratios
    a_b_ratio = [10, 5, 2, 1, 0.5, 0.2, 0.1]
    C_a_rectangle = [1.14, 1.21, 1.36, 1.51, 1.70, 1.98, 2.23]

    # Added mass coefficient for small floater, vertical (using interpolation)
    a_b_ratio_small = W_small_floater / (2 * draught)
    list_loc = 0
    while a_b_ratio_small < a_b_ratio[list_loc]:
        list_loc += 1
    lower_bound = a_b_ratio[list_loc - 1]
    higher_bound = a_b_ratio[list_loc]
    gap = higher_bound - lower_bound
    C_a_local_small_v = (higher_bound - a_b_ratio_small) / gap * C_a_rectangle[list_loc - 1] \
                        + (a_b_ratio_small - lower_bound) / gap * C_a_rectangle[list_loc]

    # Added mass coefficient for large floater, vertical (using interpolation)
    a_b_ratio_large = W_large_floater / (2 * draught)
    list_loc = 0
    while a_b_ratio_large < a_b_ratio[list_loc]:
        list_loc += 1
    lower_bound = a_b_ratio[list_loc - 1]
    higher_bound = a_b_ratio[list_loc]
    gap = higher_bound - lower_bound
    C_a_local_large_v = (higher_bound - a_b_ratio_large) / gap * C_a_rectangle[list_loc - 1] \
                        + (a_b_ratio_large - lower_bound) / gap * C_a_rectangle[list_loc]




    # Data taken from DNV report, added mass coefficients for certain a/b ratios
    a_b_ratio_rot = [0.001, 0.1, 0.2, 0.5, 1]
    C_a_rectangle_rot = [0.125, 0.147, 0.15, 0.15, 0.234]


    # Added mass rotation coefficient for small floater, rotation about z (using interpolation)
    a_b_ratio_small_rot_z = W_small_floater / L_small_floater
    list_loc = 0
    while a_b_ratio_small_rot_z > a_b_ratio_rot[list_loc]:
        list_loc += 1
    lower_bound = a_b_ratio_rot[list_loc - 1]
    higher_bound = a_b_ratio_rot[list_loc]
    gap = higher_bound - lower_bound
    C_a_local_small_rot_z = (higher_bound - a_b_ratio_small_rot_z) / gap * C_a_rectangle_rot[list_loc - 1] \
                        + (a_b_ratio_small_rot_z - lower_bound) / gap * C_a_rectangle_rot[list_loc]

    # Added mass rotation coefficient for large floater, rotation about z (using interpolation)
    a_b_ratio_large_rot_z = W_large_floater / L_large_floater
    list_loc = 0
    while a_b_ratio_large_rot_z > a_b_ratio_rot[list_loc]:
        list_loc += 1
    lower_bound = a_b_ratio_rot[list_loc - 1]
    higher_bound = a_b_ratio_rot[list_loc]
    gap = higher_bound - lower_bound
    C_a_local_large_rot_z = (higher_bound - a_b_ratio_large_rot_z) / gap * C_a_rectangle_rot[list_loc - 1] \
                        + (a_b_ratio_large_rot_z - lower_bound) / gap * C_a_rectangle_rot[list_loc]



    if Three_Dimensional_Calculation == True:
        M_added = np.zeros((6 * segments, 6 * segments))
        MR = 0
    else:
        M_added = np.zeros((3, 3))
        MR = 1


    for i in range (len(M_added) + MR):

        if Three_Dimensional_Calculation == True:
            if i % 6 == 0:
                M_added[i][i] += 0


        if i % 6 == 1:

            M_added[i - MR, i - MR] += C_Added_Mass_Pointed_Tip_Reduction * \
                                      (N_small_floater * C_a_local_small_h * 2 * draught * W_small_floater * \
                                      L_small_floater *  (0 + rho)/2. \
                                      + N_large_floater * C_a_local_large_h * 2 * draught * W_large_floater \
                                      * L_large_floater * (0 + rho)/2.)

            M_added[i - MR, i + 2 - MR] = M_added[i - MR, i - MR] * (-z_location_bottom_floater-0.5*draught)
            M_added[i + 2 - MR, i - MR] = M_added[i - MR, i + 2 - MR]

        if i % 6 == 2:
            if ExperimentalAddition == False:

                M_added[i - MR][i - MR] += N_small_floater * L_small_floater * C_a_local_small_v * np.pi * \
                                          (W_small_floater / 2.) **2 * (0 + rho) / 2. \
                                          + N_large_floater * L_large_floater * C_a_local_large_v * np.pi \
                                          * (W_large_floater / 2.) **2 * (0 + rho) / 2.

            else:
                Heave_Fit_Data = [0.070403, -0.31351708, 1.92620074, 0.]
                AM_Heave_Coeff_Small = (Heave_Fit_Data[0] + Heave_Fit_Data[1] * (W_small_floater / L_small_floater) +
                                        Heave_Fit_Data[2] * (W_small_floater / L_small_floater) ** 2 + Heave_Fit_Data[
                                            3] * (W_small_floater / L_small_floater) ** 3) * L_small_floater / draught
                AM_Heave_Coeff_Large = (Heave_Fit_Data[0] + Heave_Fit_Data[1] * (W_large_floater / L_large_floater) +
                                        Heave_Fit_Data[2] * (W_large_floater / L_large_floater) ** 2 + Heave_Fit_Data[
                                            3] * (W_large_floater / L_large_floater) ** 3) * L_large_floater / draught

                M_added[
                    i - MR, i - MR] = AM_Heave_Coeff_Small * A_small_floater * draught * rho * N_small_floater + AM_Heave_Coeff_Large * A_large_floater * draught * rho * N_large_floater

        if i % 6 == 3:

            if ExperimentalAddition == False:
                Length_small = L_small_floater + 2 * W_small_floater / 2
                Length_large = L_large_floater + W_large_floater / 2

                y_loc_coordinate_small = np.linspace(-Length_small / 2 + Length_small / 28,
                                                     Length_small / 2 - Length_small / 28, 14)
                y_loc_coordinate_large = np.linspace(-Length_large / 2 + Length_large / 28,
                                                     Length_large / 2 - Length_large / 28,
                                                     14)

                L_segment_small = (Length_small / 14)
                L_segment_large = (Length_large / 14)


                for j in range (14):
                    M_added[i - MR][i - MR] += N_small_floater * L_segment_small * C_a_local_small_v * np.pi * \
                                          (W_small_floater / 2.) **2 * (0 + rho) / 2. * y_loc_coordinate_small[j]**2 \
                                               * C_Added_Mass_Not_Infinite_Factor
                    M_added[i - MR][i - MR] += N_large_floater * L_segment_large * C_a_local_large_v * np.pi \
                                          * (W_large_floater / 2.) **2 * (0 + rho) / 2. * y_loc_coordinate_large[j]**2 \
                                               * C_Added_Mass_Not_Infinite_Factor

            else:
                Roll_Fit_Data = [-4.25163184e-04, 6.34744317e-02, -2.69940458e-01, 0.00000000e+00, 6.67155225e+00,
                                 3.41310165e+00, 3.23809138e+00, 0.00000000e+00]

                AM_Roll_Coeff_Small_1 = ((Roll_Fit_Data[0] + Roll_Fit_Data[1] * (W_small_floater / L_small_floater) +
                                          Roll_Fit_Data[2] * (
                                                      W_small_floater / L_small_floater) ** 2) * L_small_floater / draught)
                AM_Roll_Coeff_Small_2 = ((Roll_Fit_Data[4] + Roll_Fit_Data[5] * (draught / W_small_floater) +
                                          Roll_Fit_Data[6] * (draught / W_small_floater) ** 2 + Roll_Fit_Data[7] * (
                                                      draught / W_small_floater) ** 3) * W_small_floater / L_small_floater)

                AM_Roll_Coeff_Large_1 = ((Roll_Fit_Data[0] + Roll_Fit_Data[1] * (W_large_floater / L_large_floater) +
                                          Roll_Fit_Data[2] * (
                                                      W_large_floater / L_large_floater) ** 2) * L_large_floater / draught)
                AM_Roll_Coeff_Large_2 = ((Roll_Fit_Data[4] + Roll_Fit_Data[5] * (draught / W_large_floater) +
                                          Roll_Fit_Data[6] * (draught / W_large_floater) ** 2 + Roll_Fit_Data[7] * (
                                                      draught / W_large_floater) ** 3) * W_large_floater / L_large_floater)

                M_added[
                    i - MR, i - MR] = AM_Roll_Coeff_Small_1 * A_small_floater * draught * L_small_floater ** 2 * rho * N_small_floater \
                                      + AM_Roll_Coeff_Small_2 * A_small_floater * draught * (
                                                  z_location_bottom_floater + draught / 2.) ** 2 * rho * N_small_floater \
                                      + AM_Roll_Coeff_Large_1 * A_large_floater * draught * L_large_floater ** 2 * rho * N_large_floater \
                                      + AM_Roll_Coeff_Large_2 * A_large_floater * draught * (
                                                  z_location_bottom_floater + draught / 2.) ** 2 * rho * N_large_floater

    if Three_Dimensional_Calculation == True:
            if i % 6 == 4:

                # distance small floater and outer large floater to center
                dist_small_floater = (Length_segment - 2 * W_small_floater - free_space_between_segments) / 2.  # m

                M_added[i - MR, i - MR] += N_small_floater * C_a_local_small_v * L_small_floater * np.pi * \
                                           (W_small_floater/2.)**2 * (0 + rho) / 2. * dist_small_floater**2

                for j in range (N_large_floater):
                    dist_large_floater = dist_small_floater - (j+1) * (2 * dist_small_floater) \
                                         / (N_large_floater + 1 )  # m

                    M_added[i - MR, i - MR] += C_a_local_large_v * L_large_floater * np.pi * (
                                W_large_floater / 2.) ** 2 * (0 + rho) / 2. * dist_large_floater ** 2




            if i % 6 == 5:

                # distance small floater and outer large floater to center
                dist_small_floater = (Length_segment - 2 * W_small_floater - free_space_between_segments) / 2.  # m

                M_added[i - MR, i - MR] += C_Added_Mass_Pointed_Tip_Reduction * ( N_small_floater * C_a_local_small_h
                                            * 2 * draught * W_small_floater * L_small_floater * (0 + rho)/2.
                                            * dist_small_floater **2)

                for j in range (N_large_floater):
                    dist_large_floater = dist_small_floater - (j+1) * (2 * dist_small_floater) \
                                         / (N_large_floater + 1 )  # m

                    M_added[i - MR, i - MR] += C_Added_Mass_Pointed_Tip_Reduction * ( C_a_local_large_h * 2 * draught
                                                * W_large_floater * L_large_floater * (0 + rho)/2.
                                                                                      * dist_large_floater **2)

                M_added[i - MR, i - MR] = C_Added_Mass_Not_Infinite_Factor * (C_a_local_small_rot_z * N_small_floater *
                                        2 * draught * (L_small_floater / 2.) **4 * np.pi * (0 + rho)/2.
                                        + C_a_local_large_rot_z * N_large_floater * W_large_floater *
                                        (L_large_floater / 2.) **4 * np.pi * (0 + rho)/2.)

    M_added = M_added * C_Marine_Growth_Extra

    return (M_added)

def Added_turbine(characteristics_large_floater, characteristics_small_floater, M_added_tur_hori, M_added_tur_vert,M_added_tur_rot,z_location_turbines):
    
    N_turbines = (characteristics_large_floater[0] + characteristics_small_floater[0] - 1)*2
    
    M_added = np.zeros((3, 3))
    M_added[0, 0] = M_added_tur_hori * N_turbines
    M_added[1, 1] = M_added_tur_vert * N_turbines
    M_added[2, 2] = M_added_tur_rot * N_turbines

    return (M_added)

def Added_SwayPlate(Sway_plate_char,Sway_Plate_Force, Sway_Plate_Force_Design3, M_added_sway_plateD3, draught):
    
    Sway_width, Sway_height, z_location_sway, C_drag_sway, M_added_sway_plate, sway_plate_num = Sway_plate_char
    
    if Sway_Plate_Force == True:
        M_added = np.zeros((3, 3))
        M_added[0, 0] = M_added_sway_plate * sway_plate_num
        M_added[1, 1] = 0
        M_added[2, 2] = M_added_sway_plate * z_location_sway**2 * sway_plate_num
        if Sway_Plate_Force_Design3 == True:
            M_added[0, 0] = M_added_sway_plateD3
            M_added[1, 1] = 0
            M_added[2, 2] = M_added_sway_plateD3 * (draught + 9./2.)**2
    else:
        M_added = np.zeros((3, 3))

    return (M_added)

def Added_HeavePlate(Heave_plate_char, Heave_Plate_Force):
    
    Heave_dist_x,Heave_dist_y,z_location_heave, heave_plate_bottom_dist, C_drag_sway,M_added_heave_plate, M_added_heave_plate_roll, M_added_heave_plate_2, M_added_heave_plate_roll_2, second_level_heave, height_second_level, n_plates = Heave_plate_char
    
    if Heave_Plate_Force == True:
        M_added = np.zeros((3, 3))
        M_added[0, 0] = 0
        M_added[1, 1] = M_added_heave_plate
        M_added[2, 2] = M_added_heave_plate_roll
        if second_level_heave == True:
             M_added[0, 0] = 0
             M_added[1, 1] += M_added_heave_plate_2
             M_added[2, 2] += M_added_heave_plate_roll_2
#            M_added[0, 0] = 0 #this is for the upper deck only
#            M_added[1, 1] = M_added_heave_plate_2 #this is for the upper deck only
#            M_added[2, 2] = M_added_heave_plate_roll_2 #this is for the upper deck only
    else:
        M_added = np.zeros((3, 3))

    return (M_added)



def Aqwa_Damping (segments, Frequency , Frequency_damping_dict , Frequencies_list,  Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the matrix for the radiation damping according to the Ansys model

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Frequency									= Vibration frequency of the segment						(1)
# Frequency_damping_dict					= Ansys dict contaning damping matrix for various frequency	(M:6x6)
# Frequencies_list							= Ansys list of frenquecies for evaluated segment			(M)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# C_added									= Radiation damping matrix									(NxN)
    """

    # Making empty arrays and dictionaries for storage damping matrices
    Number_of_frequencies = len(Frequencies_list)

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
        Damping_frequency = np.zeros((6, 6))
    else:
        N_matrix = 3
        Damping_frequency = np.zeros((3, 3))

    # Lineair interpolation of asked frequency and available frequencies
    for i in range(Number_of_frequencies):
        if Frequency < Frequencies_list[0]:
            Damping_frequency = Frequency_damping_dict[str(0)]
        elif Frequency <= Frequencies_list[i]:
            Frequency_dif = Frequencies_list[i] - Frequencies_list[i - 1]
            Frequency_ratio_under = (Frequency - Frequencies_list[i - 1]) / Frequency_dif
            Frequency_ratio_above = (Frequencies_list[i] - Frequency) / Frequency_dif

            Damping_frequency = Frequency_damping_dict[str(i - 1)] * Frequency_ratio_above + \
                                Frequency_damping_dict[str(i)] * Frequency_ratio_under
            break

    # Create empty matrix
    C_added = np.zeros((N_matrix, N_matrix))

    # fill matrix (for every segment in 3D) with the interpolation estimation
    if Three_Dimensional_Calculation == True:
        for i in range (segments):
            for j in range (6):
                for k in range (6):
                    C_added[6*i + j, 6*i + k] = Damping_frequency[j,k]
    else:
        for j in range (3):
            for k in range(3):
                C_added[j, k] = Damping_frequency[j, k]

    # #ignore crossterms
    # for i in range (len(C_added)):
    #     for j in range (len(C_added)):
    #         if i != j:
    #             C_added[j,i] = 0


    # ignore surge terms
    if Three_Dimensional_Calculation == True:
        for j in range (segments):
            for i in range (len(C_added)):
                C_added [6*j,i] = 0
                C_added [i,6*j] = 0


    return (C_added)




def Aqwa_Mass (segments, characteristics_small_floater, characteristics_large_floater, M_segment,
               C_Marine_Growth_Extra, rho, Frequency, Frequency_Mass_dict , Frequencies_list,
               Three_Dimensional_Calculation, Ansys_char, Ansys_Aqwa_file_name):

    """"
    # Function:
# This function makes matrix for the added mass according to the Ansys model, and scales it according to the rule of
#   thumbs to allow for other sizes of the floaters and draught

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# M_segment                         		= Mass of a single segment                              	(1)
# C_Marine_Growth_Extra						= Added mass coefficient to include marine growth			(1)
# rho                               		= Density of the fluid                                  	(1)
# Frequency									= Vibration frequency of the segment						(1)
# Frequency_Mass_dict						= Ansys dict contaning mass matrix for various frequency	(M:6x6)
# Frequencies_list							= Ansys list of frenquecies for evaluated segment			(M)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Ansys_char								= Charactreristics of flaoters, M and J in Aqwa model		(3x3)

# Output:
# M_added									= Added mass matrix											(NxN)
    """

    # Making empty arrays and dictionaries for storage damping matrices
    Number_of_frequencies = len(Frequencies_list)

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
        Mass_Frequency = np.zeros((6, 6))
        MC = 0
    else:
        N_matrix = 3
        Mass_Frequency = np.zeros((3, 3))
        MC = 1

    # Linear interpolation of asked frequency and available frequencies
    for i in range(Number_of_frequencies):
        if Frequency < Frequencies_list[0]:
            Mass_Frequency = Frequency_Mass_dict[str(0)]
        elif Frequency <= Frequencies_list[i]:
            Frequency_dif = Frequencies_list[i] - Frequencies_list[i - 1]
            Frequency_ratio_under = (Frequency - Frequencies_list[i - 1]) / Frequency_dif
            Frequency_ratio_above = (Frequencies_list[i] - Frequency) / Frequency_dif

            Mass_Frequency = Frequency_Mass_dict[str(i - 1)] * Frequency_ratio_above + Frequency_Mass_dict[
                str(i)] * Frequency_ratio_under
            break
        elif Frequency > Frequencies_list[-1]:
            Mass_Frequency = Frequency_Mass_dict[str(Number_of_frequencies-1)]

    # Rescale to added mass matrix for larger
    Ansys_characteristics_small_floater , Ansys_characteristics_large_floater , Ansys_Mass = Ansys_char
    Ansys_Mass = Ansys_Mass[0]

    Ansys_N_small_floater, Ansys_W_small_floater, Ansys_L_small_floater = Ansys_characteristics_small_floater
    Ansys_N_large_floater, Ansys_W_large_floater, Ansys_L_large_floater = Ansys_characteristics_large_floater

    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1 / 2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1 / 2. * W_large_floater * 0.5 * W_large_floater  # m2

    if Ansys_Aqwa_file_name == "ANALYSIS_HOOGSTEDER.LIS":
        # Horizontal area of floaters, without tips
        Ansys_A_small_floater = Ansys_W_small_floater * Ansys_L_small_floater + 2 * 1 / 2. * Ansys_W_small_floater * Ansys_W_small_floater  # m2
        Ansys_A_large_floater = Ansys_W_large_floater * Ansys_L_large_floater + 2 * 1 / 2. * Ansys_W_large_floater * 0.5 * Ansys_W_large_floater  # m2
    else:
        # Horizontal area of floaters, including the tips
        Ansys_A_small_floater = Ansys_W_small_floater * Ansys_L_small_floater + 2 * 1 / 2. * Ansys_W_small_floater * Ansys_W_small_floater  # m2
        Ansys_A_large_floater = Ansys_W_large_floater * Ansys_L_large_floater + 2 * 1 / 2. * Ansys_W_large_floater * 0.5 * Ansys_W_large_floater  # m2

    Ansys_draught = Ansys_Mass / (( Ansys_N_small_floater * Ansys_A_small_floater + Ansys_N_large_floater * Ansys_A_large_floater) * rho )
    draught = M_segment / ((N_small_floater * A_small_floater + N_large_floater * A_large_floater) * rho )

    for i in range (len(Mass_Frequency)):

        if i == 1 - MC:
            ratio1 = draught * ( N_small_floater * W_small_floater + N_large_floater * W_large_floater ) \
                     / ( Ansys_draught * ( Ansys_N_small_floater * Ansys_W_small_floater + Ansys_N_large_floater
                                           * Ansys_W_large_floater ) )
            for j in range(len(Mass_Frequency)):
                Mass_Frequency[j,i] = Mass_Frequency[j,i] * ratio1

        if i == 2 - MC:
            ratio2 = ( N_small_floater * L_small_floater * W_small_floater**2 + N_large_floater * L_large_floater
                       * W_large_floater**2 ) / ( Ansys_N_small_floater * Ansys_L_small_floater
                                                  * Ansys_W_small_floater**2 + Ansys_N_large_floater
                                                  * Ansys_L_large_floater * Ansys_W_large_floater**2 )
            for j in range(len(Mass_Frequency)):
                Mass_Frequency[j,i] = Mass_Frequency[j,i] * ratio2

        if i == 3 - MC:
            ratio3 = (N_small_floater * L_small_floater**3 * W_small_floater**2 + N_large_floater * L_large_floater**3
                      * W_large_floater**2) / (Ansys_N_small_floater * Ansys_L_small_floater**3
                                        * Ansys_W_small_floater**2 + Ansys_N_large_floater
                                        * Ansys_L_large_floater**3 * Ansys_W_large_floater**2)
            for j in range(len(Mass_Frequency)):
                Mass_Frequency[j,i] = Mass_Frequency[j,i] * ratio3


    # Create empty matrix
    M_added = np.zeros((N_matrix, N_matrix))

    # fill matrix (for every sement in 3D) with the interpolation estimation
    if Three_Dimensional_Calculation == True:
        for i in range (segments):
            for j in range (6):
                for k in range (6):
                    M_added[6*i + j, 6*i + k] = Mass_Frequency[j,k] * C_Marine_Growth_Extra
    else:
        for j in range (3):
            for k in range(3):
                M_added[j, k] = Mass_Frequency[j, k] * C_Marine_Growth_Extra

    # #ignore crossterms
    # for i in range (len(M_added)):
    #     for j in range (len(M_added)):
    #         if i != j:
    #             M_added[j,i] = 0

    # ignore surge terms
    if Three_Dimensional_Calculation == True:
        for j in range (segments):
            for i in range (len(M_added)):
                M_added [6*j,i] = 0
                M_added [i,6*j] = 0


    return (M_added)