import numpy as np
from matplotlib import pyplot as plt
from matplotlib import animation
import matplotlib.image as mpimg

# import matplotlib
# matplotlib.use('Agg')

FPS = 16 #16 is more or less compare to the real situation. 1 second is 1 second in the animation.

plt.rcParams.update({'font.size': 18})

def Animation (Total_time,x_list,steps,characteristics_small_floater,characteristics_large_floater,M_segment,
               z_location_bottom_floater,z_location_deck,Angle_Pendulum_Initial_Tidal,Pendulum_hinge_top_location,Pendulum_hinge_bottom_location,
               Wind_Distributed_Load_Deck,Tidal_level,flow_velocity,Wave_hlp,depth,rho,smooth_time,Wind_Deck_Force,Wave_Force,Current_Force,Smooth_Start,
               slosh_kin_list, Approaching_Wave_Field, Slosh_char, Slosh_Force, draught):

    """"
    # Total_time								= Total time that is simulated by the model					(1)
    # x_list            						= Storage of location as function of time               	((steps + 1) x N)
    # steps                             		= Number of steps that the dynamic model will perform   	(1)
    # characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
    # characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
    # M_segment                         		= Mass of a single segment                              	(1)
    # z_location_bottom_floater         		= Z-coordinate of the bottom side of the floaters       	(1)
    # z_location_deck							= Z-coordinate of the middle deck							(1)
    # Angle_Pendulum_Initial_Tidal				= List of angles of pendulum still in water at given tide	(2*segments-2)
    # Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
    # Pendulum_hinge_bottom_location			= Y- and z-coordinate of the bottom hing of pendulum    	(2)
    # Wind_Distributed_Load_Deck				= Distributed load by the wind on the deck					(1)
    # Tidal_level								= Tidal level to be evaluated								(1)
    # flow_velocity                     		= Flow velocity of the fluid                            	(1)
    # Wave_hlp									= Regular wave height, length and period					(3)
    # depth										= Local water depth											(1)
    # rho                               		= Density of the fluid                                  	(1)
    # smooth_time								= Initial sinewave smoothing time							(1)
    # Wind_Deck_Force							= Option to implement force by wind on deck					(T/F)
    # Wave_Force								= Option to implement force by waves on floaters			(T/F)
    # Current_Force								= Option to implement force by current on floaters			(T/F)
    # Smooth_Start								= Option to implement smoothing of forces first 10 seconds	(T/F)
    """

        # Convert 3x3 matrix back to general 6x6 matrix for plots

    x_list_temp = np.zeros((steps + 1, 7))

    for i in range (steps + 1):
        x_list_temp[i, 1] = x_list[i, 0]
        x_list_temp[i, 2] = x_list[i, 1]
        x_list_temp[i, 3] = x_list[i, 2]
        x_list_temp[i, 4] = slosh_kin_list[i, 0] #slosh-y
        x_list_temp[i, 5] = slosh_kin_list[i, 3] #slosh-z
        x_list_temp[i, 6] = slosh_kin_list[i, 5] #dz_dy_Slosh

    x_list = x_list_temp

        # x_Slosh_y = slosh_kin_list[i, 0]
        # x_Slosh_z = slosh_kin_list[i, 3]
        # dz_dy_Slosh = slosh_kin_list[i, 5]
    S_l, S_w_l, Slosh_mass = Slosh_char

    # x_list[i,4]
    # x_list[i,5]
    # x_list[i,6]
    # print (Pendulum_hinge_bottom_location)
    # print (Pendulum_hinge_top_location)

    # seperate data from arrays which were imported
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1 / 2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1 / 2. * W_large_floater * 0.5 * W_large_floater  # m2

    # draught of structure, M / A. and relative velocity
    # draught = M_segment / ((A_small_floater * N_small_floater + A_large_floater * N_large_floater) * rho)

    # top_floater = (z_location_bottom_floater + z_location_deck) / 2
    floater_height = 6.55
    top_floater = z_location_bottom_floater + floater_height
    W_deck = L_small_floater / 1.8

    # x_Slosh_y = slosh_kin_list[i, 0]
    # x_Slosh_z = slosh_kin_list[i, 3]
    # dz_dy_Slosh = slosh_kin_list[i, 5]
    # S_l, S_w_l, Slosh_mass = Slosh_char

    s_m = 0.3 #m a small margin to have the lines of the slosh damper not over the other lines
    
    for j in range(len(x_list)):
        x_list[j, 2] += Tidal_level
        x_list[j, 1] += -Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1])

    if Current_Force == False:
        flow_velocity = 0

    if Wave_Force == True:

        Wave_height, Wave_length, Wave_period = Wave_hlp


        # Fix wave direction, as negative waves are entered for negative direction
        if Wave_height < 0:
            Wave_height = - Wave_height
            Wave_direction = -1
        else:
            Wave_direction = 1

        # Wave parameters
        Wave_k = 2 * np.pi / Wave_length
        Wave_w = 2 * np.pi / Wave_period
        a = Wave_height / 2.
    else:
        Wave_k = 2 * np.pi / 100
        Wave_w = 2 * np.pi / 8
        a = 0
        Wave_direction = 1
        Wave_height = 0
        Wave_length = 10

    x_list_ani = np.zeros((FPS * Total_time,7))
    for i in range (FPS * Total_time):
        Time_ani = i / FPS #Is the time in seconds
        x_list_low = int(np.floor(steps / Total_time * Time_ani))
        x_list_real = steps / Total_time * Time_ani
        x_list_high = int(np.floor(steps / Total_time * Time_ani) + 1)
        
        x_list_ani[i] = x_list[x_list_low] * (1 - (x_list_real - x_list_low)) + x_list[x_list_high] * ((x_list_real - x_list_low))

    x_list = x_list_ani

    for j in range (len(x_list)):
        x_list[j,2] = x_list[j,2] - z_location_bottom_floater - draught

    smooth_function_value = np.zeros(int(FPS * smooth_time))
    for i in range(int(FPS * smooth_time)):
        smooth_function_value[i] = 0.5 - 0.5 * np.cos(np.pi / len(smooth_function_value) * i)



    fig = plt.figure(figsize=(10,5.4))
    ax = plt.axes(xlim=(-1.5*L_small_floater, 1.5*L_small_floater), ylim=(-depth*1.25, (3*L_small_floater/9.5*5.2)-depth*1.25))
    plt.ylabel("Z-coordinate [m]")
    plt.xlabel("Y-coordinate [m]")

    plt.hlines(Pendulum_hinge_bottom_location[1][1]+Tidal_level- z_location_bottom_floater - draught,-1.5*L_small_floater, 1.5*L_small_floater, color= 'brown', lw=2)

    Xarrow = np.linspace(-1.5*L_small_floater, 1.5*L_small_floater,12)
    Yarrow = np.linspace(-depth*1.25, (3*L_small_floater/9.5*5.2)-depth*1.25, 20)

    if Current_Force == True:
        if flow_velocity < 0:
            DirectionFlow = -1
        else:
            DirectionFlow = 1

    if Wind_Deck_Force == True:
        if Wind_Distributed_Load_Deck < 0:
            DirectionWind = -1
        else:
            DirectionWind = 1

    for k in range(len(Xarrow)):
        for j in range(len(Yarrow)):
            if k == 0 or j == 0 or k == len(Xarrow) - 1 or j == len(Yarrow) - 1:
                continue
            if Yarrow[j] < Pendulum_hinge_bottom_location[1][1]+Tidal_level- z_location_bottom_floater - draught + 1:
                continue
            if Yarrow[j] < Tidal_level - 0.5* abs(Wave_height)-0.5:
                if Current_Force == True and flow_velocity != 0:
                    plt.arrow(Xarrow[k], Yarrow[j], 1.6 * DirectionFlow, 0, head_width=0.5, head_length=0.5, color='blue', alpha=0.25)
            if Yarrow[j] > Tidal_level + 0.5 * abs(Wave_height)+0.5:
                if Wind_Deck_Force == True:
                    plt.arrow(Xarrow[k], Yarrow[j], 1.6 * DirectionWind, 0, head_width=0.5, head_length=0.5, color='grey', alpha=0.25)


    line1, = ax.plot([], [], lw=4)
    line2, = ax.plot([], [], lw=2)
    line3, = ax.plot([], [], lw=2)
    line4, = ax.plot([], [], lw=2)
    line5, = ax.plot([], [], lw=2)
    line6, = ax.plot([], [], lw=2)
    line7, = ax.plot([], [], lw=2)
    line8, = ax.plot([], [], lw=2)
    line9, = ax.plot([], [], lw=2)
    line10, = ax.plot([], [], lw=2)
    line11, = ax.plot([], [], lw=2)
    line12, = ax.plot([], [], lw=2)
    line14, = ax.plot([], [], lw=2)
    line15, = ax.plot([], [], lw=2)
    line16, = ax.plot([], [], lw=2)
    line17, = ax.plot([], [], lw=2)
    line18, = ax.plot([], [], lw=2)
    line19, = ax.plot([], [], lw=1)
    line20, = ax.plot([], [], lw=1)
    line21, = ax.plot([], [], lw=1)
    line13, = ax.plot([], [], lw=2)

    def init():
        line1.set_data([], [])
        line2.set_data([], [])
        line3.set_data([], [])
        line4.set_data([], [])
        line5.set_data([], [])
        line6.set_data([], [])
        line7.set_data([], [])
        line8.set_data([], [])
        line9.set_data([], [])
        line10.set_data([], [])
        line11.set_data([], [])
        line12.set_data([], [])
        line14.set_data([], [])
        line15.set_data([], [])
        line16.set_data([], [])
        line17.set_data([], [])
        line18.set_data([], [])
        line19.set_data([], [])
        line20.set_data([], [])
        line21.set_data([], [])
        line13.set_data([], [])
        return line1,line2,line3,line4,line5,line6,line7,line8,line9,line10,line11,line12,line14,line15,line16,line17,line18,line19,line20,line21, line13

    def animate(i):
        Left_Deck = [-W_deck/2 + x_list[i,1] - x_list[i,3] * z_location_deck ,z_location_deck + x_list[i,2] - x_list[i,3] * W_deck / 2]
        Right_Deck = [W_deck/2 + x_list[i,1] - x_list[i,3] * z_location_deck ,z_location_deck + x_list[i,2] + x_list[i,3] * W_deck / 2]

        Top_Connect_Left = [-W_deck/3 + x_list[i,1] - x_list[i,3] * z_location_deck ,z_location_deck + x_list[i,2] - x_list[i,3] * W_deck / 3]
        Bottom_Connect_Left =  [-W_deck/3*2 + x_list[i,1] - x_list[i,3] * top_floater,top_floater + x_list[i,2] - x_list[i,3] * W_deck / 3*2]

        Top_Connect_Right = [W_deck/3 + x_list[i,1] - x_list[i,3] * z_location_deck ,z_location_deck + x_list[i,2] + x_list[i,3] * W_deck / 3]
        Bottom_Connect_Right = [W_deck/3*2 + x_list[i,1] - x_list[i,3] * top_floater ,top_floater + x_list[i,2] + x_list[i,3] * W_deck / 3*2]

        Bottom_Connect_Middle = [W_deck/3*0 + x_list[i,1] - x_list[i,3] * top_floater ,top_floater + x_list[i,2] + x_list[i,3] * W_deck / 3*0]

        Left_Top_Floater = [-L_small_floater/2 + x_list[i,1] - x_list[i,3] * top_floater ,top_floater + x_list[i,2] - x_list[i,3] * L_small_floater / 2]
        Right_Top_Floater = [L_small_floater/2 + x_list[i,1] - x_list[i,3] * top_floater ,top_floater + x_list[i,2] + x_list[i,3] * L_small_floater / 2]

        Left_Bottom_Floater = [-L_small_floater/2 + x_list[i,1] - x_list[i,3] * z_location_bottom_floater ,z_location_bottom_floater + x_list[i,2] - x_list[i,3] * L_small_floater / 2]
        Right_Bottom_Floater = [L_small_floater/2 + x_list[i,1] - x_list[i,3] * z_location_bottom_floater ,z_location_bottom_floater + x_list[i,2] + x_list[i,3] * L_small_floater / 2]

        Top_Pendulum = [Pendulum_hinge_top_location[0] + x_list[i,1] - x_list[i,3] * Pendulum_hinge_top_location[1] ,Pendulum_hinge_top_location[1] + x_list[i,2] + x_list[i,3] * Pendulum_hinge_top_location[0]]
        Bottom_Pendulum = [Pendulum_hinge_bottom_location[0][1]-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]+Tidal_level- z_location_bottom_floater - draught]

        Top_Pendulum2 = [Pendulum_hinge_top_location[0] + W_deck/30 + x_list[i,1] - x_list[i,3] * z_location_bottom_floater ,z_location_bottom_floater + x_list[i,2] + x_list[i,3] * (Pendulum_hinge_top_location[0] + W_deck/30)]
        Top_Pendulum3 = [Pendulum_hinge_top_location[0] - W_deck/30 + x_list[i,1] - x_list[i,3] * z_location_bottom_floater ,z_location_bottom_floater + x_list[i,2] + x_list[i,3] * (Pendulum_hinge_top_location[0] - W_deck/30)]
        Top_Pendulum4 = [Pendulum_hinge_top_location[0] + x_list[i, 1] - x_list[i, 3] * (Pendulum_hinge_top_location[1] - (z_location_bottom_floater - Pendulum_hinge_top_location[1])), Pendulum_hinge_top_location[1] - (z_location_bottom_floater - Pendulum_hinge_top_location[1]) + x_list[i, 2] + x_list[i, 3] * Pendulum_hinge_top_location[0]]

        SoilmarkTop1 = [-1+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]+Tidal_level- z_location_bottom_floater - draught]
        SoilmarkTop2 = [1+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]+Tidal_level- z_location_bottom_floater - draught]

        SoilmarkBottom1 = [-3+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]-2+Tidal_level- z_location_bottom_floater - draught]
        SoilmarkBottom2 = [-1+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]-2+Tidal_level- z_location_bottom_floater - draught]
        SoilmarkBottom3 = [1+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]-2+Tidal_level- z_location_bottom_floater - draught]
        SoilmarkBottom4 = [3+L_small_floater-Tidal_level * np.tan(Angle_Pendulum_Initial_Tidal[1]),Pendulum_hinge_bottom_location[1][1]-2+Tidal_level- z_location_bottom_floater - draught]
        
        
        
        
        
        if Slosh_Force==True:
            if x_list[i,6] <= 2*S_w_l/S_l and x_list[i,6] >= -2*S_w_l/S_l:
                Left_Slosh = [-S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + S_w_l - x_list[i,6] * S_l/2. + x_list[i,2] - x_list[i,3] * S_l/2.]
                Right_Slosh = [S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + S_w_l + x_list[i,6] * S_l/2. + x_list[i,2] + x_list[i,3] * S_l/2.]
            elif x_list[i,6] > 2*S_w_l/S_l:
                Left_Slosh = [S_l/2.-(S_l/2.- x_list[i,4])*3 + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + x_list[i,2] - x_list[i,3] * S_l/2.]
                Right_Slosh = [S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + (S_w_l/2.+x_list[i,5])*3  + x_list[i,2] + x_list[i,3] * S_l/2.]
            else:
                Left_Slosh = [-S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + (S_w_l/2.+x_list[i,5])*3  + x_list[i,2] - x_list[i,3] * S_l/2.]
                Right_Slosh = [-S_l/2.+(S_l/2.- abs(x_list[i,4]))*3 + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + S_w_l/2.),s_m + z_location_bottom_floater + x_list[i,2] + x_list[i,3] * S_l/2.]
         
            Slosh_wall_left_1 = [-S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater),s_m + z_location_bottom_floater + x_list[i,2] + x_list[i,3] * -S_l/2.]
            Slosh_wall_left_2 = [-S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + floater_height - s_m),s_m + z_location_bottom_floater + floater_height - s_m + x_list[i,2] + x_list[i,3] * -S_l/2.]
            
            Slosh_wall_right_1 = [S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater),s_m + z_location_bottom_floater + x_list[i,2] + x_list[i,3] * S_l/2.]
            Slosh_wall_right_2 = [S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater + floater_height - s_m),s_m + z_location_bottom_floater + floater_height - s_m + x_list[i,2] + x_list[i,3] * S_l/2.]
            
            Slosh_bottom_1 = [-S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater),s_m + z_location_bottom_floater + x_list[i,2] + x_list[i,3] * -S_l/2.]
            Slosh_bottom_2 = [S_l/2. + x_list[i,1] - x_list[i,3] * (s_m + z_location_bottom_floater),s_m + z_location_bottom_floater + x_list[i,2] + x_list[i,3] * S_l/2.]
            
            y_Slosh = np.linspace(Left_Slosh[0],Right_Slosh[0],2)
            z_Slosh = np.linspace(Left_Slosh[1],Right_Slosh[1],2)
            
            y_Slosh_wall_left = np.linspace(Slosh_wall_left_1[0],Slosh_wall_left_2[0],2)
            z_Slosh_wall_left = np.linspace(Slosh_wall_left_1[1],Slosh_wall_left_2[1],2)
            
            y_Slosh_wall_right = np.linspace(Slosh_wall_right_1[0],Slosh_wall_right_2[0],2)
            z_Slosh_wall_right = np.linspace(Slosh_wall_right_1[1],Slosh_wall_right_2[1],2)
            
            y_Slosh_bottom = np.linspace(Slosh_bottom_1[0],Slosh_bottom_2[0],2)
            z_Slosh_bottom = np.linspace(Slosh_bottom_1[1],Slosh_bottom_2[1],2)

        y_Deck = np.linspace(Left_Deck[0],Right_Deck[0],2)
        z_Deck = np.linspace(Left_Deck[1],Right_Deck[1],2)

        y_Connect_Left = np.linspace(Top_Connect_Left[0],Bottom_Connect_Left[0],2)
        z_Connect_left = np.linspace(Top_Connect_Left[1],Bottom_Connect_Left[1],2)

        y_Connect_Right = np.linspace(Top_Connect_Right[0],Bottom_Connect_Right[0],2)
        z_Connect_Right = np.linspace(Top_Connect_Right[1],Bottom_Connect_Right[1],2)

        y_Connect_D1 = np.linspace(Top_Connect_Left[0],Bottom_Connect_Middle[0],2)
        z_Connect_D1 = np.linspace(Top_Connect_Left[1],Bottom_Connect_Middle[1],2)

        y_Connect_D2 = np.linspace(Top_Connect_Right[0],Bottom_Connect_Middle[0],2)
        z_Connect_D2 = np.linspace(Top_Connect_Right[1],Bottom_Connect_Middle[1],2)

        y_top_floater = np.linspace(Left_Top_Floater[0],Right_Top_Floater[0],2)
        z_top_floater = np.linspace(Left_Top_Floater[1],Right_Top_Floater[1],2)

        y_left_floater = np.linspace(Left_Top_Floater[0],Left_Bottom_Floater[0],2)
        z_left_floater = np.linspace(Left_Top_Floater[1],Left_Bottom_Floater[1],2)

        y_right_floater = np.linspace(Right_Bottom_Floater[0],Right_Top_Floater[0],2)
        z_right_floater = np.linspace(Right_Bottom_Floater[1],Right_Top_Floater[1],2)

        y_bottom_floater = np.linspace(Left_Bottom_Floater[0],Right_Bottom_Floater[0],2)
        z_bottom_floater = np.linspace(Left_Bottom_Floater[1],Right_Bottom_Floater[1],2)

        y_pendulum = np.linspace(Top_Pendulum[0],Bottom_Pendulum[0],2)
        z_pendulum = np.linspace(Top_Pendulum[1], Bottom_Pendulum[1], 2)

        y_pendulum2 = np.linspace(Top_Pendulum2[0],Top_Pendulum4[0],2)
        z_pendulum2 = np.linspace(Top_Pendulum2[1], Top_Pendulum4[1], 2)

        y_pendulum3 = np.linspace(Top_Pendulum3[0],Top_Pendulum4[0],2)
        z_pendulum3 = np.linspace(Top_Pendulum3[1], Top_Pendulum4[1], 2)

        y_soil1 = np.linspace(SoilmarkTop1[0],SoilmarkBottom1[0],2)
        z_soil1 = np.linspace(SoilmarkTop1[1], SoilmarkBottom1[1], 2)

        y_soil2 = np.linspace(SoilmarkTop1[0],SoilmarkBottom3[0],2)
        z_soil2 = np.linspace(SoilmarkTop1[1], SoilmarkBottom3[1], 2)

        y_soil3 = np.linspace(SoilmarkTop2[0],SoilmarkBottom2[0],2)
        z_soil3 = np.linspace(SoilmarkTop2[1], SoilmarkBottom2[1], 2)

        y_soil4 = np.linspace(SoilmarkTop2[0],SoilmarkBottom4[0],2)
        z_soil4 = np.linspace(SoilmarkTop2[1], SoilmarkBottom4[1], 2)


        # y_bridge = x_list[i,1]
        eta = np.zeros(201)
        y_space = np.linspace(-1.5*L_small_floater, 1.5*L_small_floater, 201)
        for j in range (201):
            eta[j] = a * np.cos(Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (
                    y_space[j])) + Wave_k * a ** 2 * np.cosh(Wave_k * depth) / (
                                      4 * (np.sinh(Wave_k * depth)) ** 3) * (
                                      2 + np.cosh(2 * Wave_k * depth)) * np.cos(
                2 * (Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space[j])))

            if Smooth_Start == True:
                if i / FPS < smooth_time:
                    eta[j] = eta[j] * smooth_function_value[i]

            if Approaching_Wave_Field == True:
                NumberOfWavesCancelled = int(np.ceil(0.3 * Total_time / Wave_period + flow_velocity * 0.3 * Total_time / Wave_length)+1)
                Cancelled_length = NumberOfWavesCancelled * Wave_length

                if Wave_direction == 1:
                    if (i/FPS / Wave_period + flow_velocity * i/FPS / Wave_length)*Wave_length - y_space[j] < Cancelled_length + 0.25 * Wave_length:
                        eta[j] = 0
                    elif (i/FPS / Wave_period + flow_velocity * i/FPS / Wave_length)*Wave_length - y_space[j] < Cancelled_length + 0.5 * Wave_length:
                        eta[j] = 0 \
                                 * (0.5 - 0.5*np.cos(2* (Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space[j])))) \
                                 + eta[j] \
                                 * (0.5+0.5 * np.cos(2*(Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space[j]))))
                else:
                    if (i/FPS / Wave_period + flow_velocity * i/FPS / Wave_length)*Wave_length * Wave_direction - y_space[j] > Cancelled_length * Wave_direction - 0.25 * Wave_length:
                        eta[j] = 0
                    elif (i/FPS / Wave_period + flow_velocity * i/FPS / Wave_length)*Wave_length * Wave_direction - y_space[j] > Cancelled_length * Wave_direction - 0.5 * Wave_length:
                        eta[j] = 0 \
                                 * (0.5 - 0.5*np.cos(2* (Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space[j])))) \
                                 + eta[j] \
                                 * (0.5+0.5 * np.cos(2*(Wave_w * i/FPS + flow_velocity * i/FPS / Wave_length * 2 * np.pi - Wave_direction * Wave_k * (y_space[j]))))

            eta[j] += Tidal_level

        line1.set_data(y_Deck, z_Deck)
        line2.set_data(y_Connect_Left, z_Connect_left)
        line3.set_data(y_Connect_Right, z_Connect_Right)
        line4.set_data(y_Connect_D1, z_Connect_D1)
        line5.set_data(y_Connect_D2, z_Connect_D2)
        line6.set_data(y_top_floater, z_top_floater)
        line7.set_data(y_left_floater, z_left_floater)
        line8.set_data(y_right_floater, z_right_floater)
        line9.set_data(y_bottom_floater, z_bottom_floater)
        line10.set_data(y_pendulum, z_pendulum)
        line11.set_data(y_pendulum2, z_pendulum2)
        line12.set_data(y_pendulum3, z_pendulum3)
        line14.set_data(y_soil1, z_soil1)
        line15.set_data(y_soil2, z_soil2)
        line16.set_data(y_soil3, z_soil3)
        line17.set_data(y_soil4, z_soil4)
        if Slosh_Force==True:
            line18.set_data(y_Slosh, z_Slosh)
            line19.set_data(y_Slosh_wall_left, z_Slosh_wall_left)
            line20.set_data(y_Slosh_wall_right, z_Slosh_wall_right)
            line21.set_data(y_Slosh_bottom, z_Slosh_bottom)
        line13.set_data(y_space, eta)


        line1.set_color('black')
        line2.set_color('black')
        line3.set_color('black')
        line4.set_color('black')
        line5.set_color('black')
        line6.set_color('black')
        line7.set_color('black')
        line8.set_color('black')
        line9.set_color('black')
        line10.set_color('black')
        line11.set_color('black')
        line12.set_color('black')
        line14.set_color('brown')
        line15.set_color('brown')
        line16.set_color('brown')
        line17.set_color('brown')
        if Slosh_Force==True:
            line18.set_color('royalblue')
            line19.set_color('grey')
            line20.set_color('grey')
            line21.set_color('grey')
        line13.set_color('cadetblue')
        return line1,line2,line3,line4,line5,line6,line7,line8,line9,line10,line11,line12,line14,line15,line16,line17,line18,line19,line20,line21,line13



    anim = animation.FuncAnimation(fig, animate, init_func=init,
                                   frames=Total_time*FPS, interval=1000/FPS, blit=True)


