import numpy as np



def Damping_Ansys_Aqwa(Three_Dimensional_Calculation, Ansys_file_name):

    """"
    # Function:
# This function calculates the added damping for the structure. It imports the data generated by Ansys Aqwa for various frequencies

# Input:
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Ansys_Aqwa_file_name						= File name which is used to scrape results from Aqwa		(str)

# Output:
# Frequency_damping_dict        = Damping matrix dictionary for specific frequencies ( N x (6 x 6) )
# Frequencies_list              = List of frequencies that are stored in dictionary (N)
"""

    # Load Ansys file
    Ansys_Analysis = open(Ansys_file_name, 'r')
    content = Ansys_Analysis.read().splitlines()
    Ansys_Analysis.close()

    Frequency_damping_dict = {}
    Added_Damp_Loc = []
    Frequency_Loc = []
    Frequencies_list = []

    # Locate all damping matrices from Ansys file
    for i in range(len(content)):
        if content[i] == "              * * * * H Y D R O D Y N A M I C   P A R A M E T E R S   F O R   S T R U C T U R E   1 * * * *":
            if content[i + 26] == "                                                   DAMPING":
                Added_Damp_Loc.append(i + 26)
                Frequency_Loc.append(i + 3)


    for i in range(len(Added_Damp_Loc)):

        # Remove strings and add to frequency list
        Frequencies_list.append(float(str.split(str(content[Frequency_Loc[i]]))[7]))
        Frequencies_list[i] = Frequencies_list[i] / 2 / np.pi

        # Filter damping matrices and into 6 times (1x6) shape
        Matrix_part_A = content[Added_Damp_Loc[i] + 7]
        Matrix_part_B = content[Added_Damp_Loc[i] + 9]
        Matrix_part_C = content[Added_Damp_Loc[i] + 11]
        Matrix_part_D = content[Added_Damp_Loc[i] + 13]
        Matrix_part_E = content[Added_Damp_Loc[i] + 15]
        Matrix_part_F = content[Added_Damp_Loc[i] + 17]

        Matrix_part_A = str.split(str(Matrix_part_A))[1:7]
        Matrix_part_B = str.split(str(Matrix_part_B))[1:7]
        Matrix_part_C = str.split(str(Matrix_part_C))[1:7]
        Matrix_part_D = str.split(str(Matrix_part_D))[1:7]
        Matrix_part_E = str.split(str(Matrix_part_E))[1:7]
        Matrix_part_F = str.split(str(Matrix_part_F))[1:7]

        # Make damping matrices into one matrix and remove strings
        if Matrix_part_A[0] != '0.0000E+00' and Matrix_part_B[1] != '0.0000E+00' and Matrix_part_C[2] != '0.0000E+00' and Matrix_part_D[
            3] != '0.0000E+00' and Matrix_part_E[4] != '0.0000E+00' and Matrix_part_F[5] != '0.0000E+00':
            Added_Damping_Matrix_frequency_str = np.array([Matrix_part_A, Matrix_part_B, Matrix_part_C, Matrix_part_D, Matrix_part_E, Matrix_part_F])

            if Three_Dimensional_Calculation == True:
                Added_Damping_Matrix_frequency = np.zeros((6, 6))
                for j in range(6):
                    for k in range(6):
                        Added_Damping_Matrix_frequency[k][j] = float(Added_Damping_Matrix_frequency_str[k][j])
            else:
                Added_Damping_Matrix_frequency = np.zeros((3, 3))
                for j in range (3):
                    for k in range(3):
                        Added_Damping_Matrix_frequency[k][j] = float(Added_Damping_Matrix_frequency_str[k+1][j+1])




        Frequency_damping_dict[str(i)] = Added_Damping_Matrix_frequency



    return (Frequency_damping_dict, Frequencies_list)
















def Mass_Ansys_Aqwa(Three_Dimensional_Calculation, Ansys_file_name):

    """"
    # Function:
# This function calculates the added mass for the structure. It imports the data generated by Ansys Aqwa for various frequencies

# Input:
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)
# Ansys_Aqwa_file_name						= File name which is used to scrape results from Aqwa		(str)

# Output:
# Frequency_Mass_dict           = Added mass matrix dictionary for specific frequencies ( N x (6 x 6) )
# Frequencies_list              = List of frequencies that are stored in dictionary (N)
    """

    # Load Ansys file
    Ansys_Analysis = open(Ansys_file_name, 'r')
    content = Ansys_Analysis.read().splitlines()
    Ansys_Analysis.close()

    Frequency_Mass_dict = {}
    Added_Damp_Loc = []
    Frequency_Loc = []
    Frequencies_list = []

    # Locate all Mass matrices from Ansys file
    for i in range(len(content)):
        if content[i] == "              * * * * H Y D R O D Y N A M I C   P A R A M E T E R S   F O R   S T R U C T U R E   1 * * * *":
            if content[i + 6] == "                                                   ADDED  MASS":
                Added_Damp_Loc.append(i + 6)
                Frequency_Loc.append(i+3)


    for i in range(len(Added_Damp_Loc)):

        # Remove strings and add to frequency list
        Frequencies_list.append(float(str.split(str(content[Frequency_Loc[i]]))[7]))
        Frequencies_list[i] = Frequencies_list[i] / 2 / np.pi

        # Filter Mass matrices and into 6 times (1x6) shape
        Matrix_part_A = content[Added_Damp_Loc[i] + 7]
        Matrix_part_B = content[Added_Damp_Loc[i] + 9]
        Matrix_part_C = content[Added_Damp_Loc[i] + 11]
        Matrix_part_D = content[Added_Damp_Loc[i] + 13]
        Matrix_part_E = content[Added_Damp_Loc[i] + 15]
        Matrix_part_F = content[Added_Damp_Loc[i] + 17]

        Matrix_part_A = str.split(str(Matrix_part_A))[1:7]
        Matrix_part_B = str.split(str(Matrix_part_B))[1:7]
        Matrix_part_C = str.split(str(Matrix_part_C))[1:7]
        Matrix_part_D = str.split(str(Matrix_part_D))[1:7]
        Matrix_part_E = str.split(str(Matrix_part_E))[1:7]
        Matrix_part_F = str.split(str(Matrix_part_F))[1:7]

        # Make Mass matrices into one matrix and remove strings
        if Matrix_part_A[0] != '0.0000E+00' and Matrix_part_B[1] != '0.0000E+00' and Matrix_part_C[2] != '0.0000E+00' and Matrix_part_D[
            3] != '0.0000E+00' and Matrix_part_E[4] != '0.0000E+00' and Matrix_part_F[5] != '0.0000E+00':
            Added_Mass_Matrix_frequency_str = np.array([Matrix_part_A, Matrix_part_B, Matrix_part_C, Matrix_part_D, Matrix_part_E, Matrix_part_F])

            if Three_Dimensional_Calculation == True:
                Added_Mass_Matrix_frequency = np.zeros((6, 6))
                for j in range(6):
                    for k in range(6):
                        Added_Mass_Matrix_frequency[k][j] = float(Added_Mass_Matrix_frequency_str[k][j])
            else:
                Added_Mass_Matrix_frequency = np.zeros((3, 3))
                for j in range (3):
                    for k in range(3):
                        Added_Mass_Matrix_frequency[k][j] = float(Added_Mass_Matrix_frequency_str[k+1][j+1])

        Frequency_Mass_dict[str(i)] = Added_Mass_Matrix_frequency



    return (Frequency_Mass_dict)
