# import of libraries
import numpy as np
from scipy import linalg as LA

def xddot_calculator (x_time, xdot_time, F_time, M_matrix, K_matrix, C_matrix):
    # Inverse M_matrix
    M_inv = np.linalg.inv(M_matrix)
    # Forces acting on the system
    Spring_force = np.dot ( K_matrix , x_time )
    Damper_force = np.dot ( C_matrix , xdot_time )
    Total_force =  F_time - Spring_force - Damper_force
    # Acceleration as F = M a
    xddot_time = np.dot( M_inv , Total_force )
    return (xddot_time)



def Natural (M_matrix, K_matrix):

    """"
    # Function:
# This function calculates the eigenfrequencies and periods of a given M and K matrix

# Input:
# M_matrix          						= mass matrix                                           	(NxN)
# K_matrix          						= spring matrix                                         	(NxN)

# Output:
# Natural_Frequencies						= List of natural frequencies of a mass-spring system		(N)
# Natural_Period							= List of eigen periods of a mass-spring system				(N)
"""

    Eigenvalues = abs(LA.eigvals(K_matrix, M_matrix))
    Natural_Period = np.zeros(len(Eigenvalues))
    Natural_Frequencies = np.zeros(len(Eigenvalues))

    for i in range (len(Eigenvalues)):
        Natural_Frequencies[i] = np.sqrt(Eigenvalues[i])
        Natural_Period[i] = 2*np.pi/Natural_Frequencies[i]

    return (Natural_Frequencies, Natural_Period)






def Angle(step, x_list, segments, Length_segment, free_space_between_segments, characteristics_small_floater,
          Pendulum_hinge_top_location, Pendulum_hinge_bottom_location, Three_Dimensional_Calculation):

    """"
    # Function:
# This function updates the angle of the pendulums based on the location of the segment

# Input:
# x_list            						= storage of location as function of time               	((steps + 1)x N)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Pendulum_hinge_bottom_location			= Y- and z-coordinate of the bottom hing of pendulum    	(2)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# step										= Current step number										(1)

# Output:
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)"""

    if Three_Dimensional_Calculation == True:
        MC = 0
    else:
        MC = 7

    W_small_floater = characteristics_small_floater[1]

    dist_pendulums = Length_segment - 2 * W_small_floater - free_space_between_segments  # m

    top_hinge_y_location = Pendulum_hinge_top_location[0]
    top_hinge_z_location = Pendulum_hinge_top_location[1]

    bottom_hinge_y_location = Pendulum_hinge_bottom_location[0]
    bottom_hinge_z_location = Pendulum_hinge_bottom_location[1]

    top_hinge_y_location_new = np.zeros(2 * segments - 2)
    top_hinge_z_location_new = np.zeros(2 * segments - 2)

    for i in range (segments):
        if Three_Dimensional_Calculation == False:
            if i == 0:
                continue
            if i == segments - 1:
                continue

        y_displacement = x_list[step][1 + 6 * i - MC]
        z_displacement = x_list[step][2 + 6 * i - MC]
        Rx_rotation = x_list[step][3 + 6 * i - MC]

        if Three_Dimensional_Calculation == True:
            Ry_rotation = x_list[step][4 + 6 * i - MC]
            Rz_rotation = x_list[step][5 + 6 * i - MC]

            # for every segment that is not the last one, so calculating the pendulum angle on
            # positive x location segment
            if i != segments - 1:
                top_hinge_y_location_new[2 * i] += y_displacement - Rx_rotation * top_hinge_z_location + Rz_rotation \
                                                   * 1/2. * dist_pendulums
                top_hinge_z_location_new[2 * i] += z_displacement + Rx_rotation * top_hinge_y_location - Ry_rotation \
                                                   * 1/2. * dist_pendulums

            # for every segment that is not the first one, so calculating the pendulum angle on
            # negative x location segment
            if i != 0:
                top_hinge_y_location_new[2 * i - 1] += y_displacement - Rx_rotation * top_hinge_z_location \
                                                       - Rz_rotation * 1/2. * dist_pendulums
                top_hinge_z_location_new[2 * i - 1] += z_displacement + Rx_rotation * top_hinge_y_location \
                                                       + Ry_rotation * 1/2. * dist_pendulums

        else:
            # for every segment that is not the last one, so calculating the pendulum angle on
            # positive x location segment
            if i != segments - 1:
                top_hinge_y_location_new[
                    2 * i] += y_displacement - Rx_rotation * top_hinge_z_location
                top_hinge_z_location_new[
                    2 * i] += z_displacement + Rx_rotation * top_hinge_y_location

            # for every segment that is not the first one, so calculating the pendulum angle on
            # negative x location segment
            if i != 0:
                top_hinge_y_location_new[
                    2 * i - 1] += y_displacement - Rx_rotation * top_hinge_z_location
                top_hinge_z_location_new[
                    2 * i - 1] += z_displacement + Rx_rotation * top_hinge_y_location


    Angle_Pendulum = np.zeros(segments * 2 - 2)

    for i in range (segments * 2 - 2):

        y_dif = top_hinge_y_location + top_hinge_y_location_new[i] - bottom_hinge_y_location[i]
        z_dif = top_hinge_z_location + top_hinge_z_location_new[i] - bottom_hinge_z_location[i]

        Angle_Pendulum[i] = np.arctan(z_dif / y_dif)



    return (Angle_Pendulum)






def Maximum(x_list, xddot_list, xddot_list_tot, Total_Time, segments, F_pendulum, Three_Dimensional_Calculation, Wave_hlp , flow_velocity):

    """"
    # Function:
# This function updates the angle of the pendulums based on the location of the segment

# Input:
# x_list            						= storage of location as function of time               	((steps + 1)x N)
# xddot_list        						= storage of acceleration as function of time           	((steps) x N)
# Total_Time								= Total time that is simulated by the model					(1)
# segments                          		= Number of segments in the bridge                      	(1)
# Three_Dimensional_Calculation				= Option to enable three dimensional calculation			(T/F)

# Output:
# Max_location								= List of maximum translations or rotations					(6)
# Max_acceleration							= List of maximum acceleration of translation or rotation   (6)
"""
    Wave_Height, Wave_Length, Wave_Period = Wave_hlp


    Max_y = 0
    Max_z = 0
    Max_Rx = 0

    Max_yddot = 0
    Max_zddot = 0
    Max_Rxddot = 0
    Max_ddot_tot = 0

    Max_F_pendulum = -10**50 * np.ones(len(F_pendulum[0]))
    Min_F_pendulum = 10**50 * np.ones(len(F_pendulum[0]))

    # Remove initial jump maximum
    steps = len(x_list)
    start_search = max(0 , int(steps - steps / Total_Time * Wave_Period * 2))

    for j in range (steps - start_search):
        j_search = j + start_search -1

        Max_y_temp = max(np.max(x_list[j_search, 0]), - np.min(x_list[j_search, 0]))
        if Max_y_temp > Max_y:
            Max_y = Max_y_temp

        Max_z_temp = max(np.max(x_list[j_search, 1]), - np.min(x_list[j_search, 1]))
        if Max_z_temp > Max_z:
            Max_z = Max_z_temp
        
        Max_Rx_temp = max(np.max(x_list[j_search, 2]), - np.min(x_list[j_search, 2]))
        if Max_Rx_temp > Max_Rx:
            Max_Rx = Max_Rx_temp


        Max_yddot_temp = max(np.max(xddot_list[j_search, 0]),- np.min(xddot_list[j_search, 0]))
        if Max_yddot_temp > Max_yddot:
            Max_yddot = Max_yddot_temp
            
        Max_zddot_temp = max(np.max(xddot_list[j_search, 1]),- np.min(xddot_list[j_search, 1]))
        if Max_zddot_temp > Max_zddot:
            Max_zddot = Max_zddot_temp
            
        Max_Rxddot_temp = max(np.max(xddot_list[j_search, 2]),- np.min(xddot_list[j_search, 2]))
        if Max_Rxddot_temp > Max_Rxddot:
            Max_Rxddot = Max_Rxddot_temp
            
        Max_ddot_tot_temp = max(np.max(xddot_list_tot[j_search]),- np.min(xddot_list_tot[j_search]))
        if Max_ddot_tot_temp > Max_ddot_tot:
            Max_ddot_tot = Max_ddot_tot_temp
            

    Max_location = [round(Max_y,4), round(Max_z,4), round(Max_Rx,5)]
    Max_acceleration = [round(Max_yddot,4), round(Max_zddot,4), round(Max_Rxddot,4), round(Max_ddot_tot,4)]

    for i in range (len(F_pendulum[0])):
        for j in range (steps-1):
            if j > start_search:
                if F_pendulum[j,i] > Max_F_pendulum[i]:
                    Max_F_pendulum[i] = round(F_pendulum[j,i])

    for i in range (len(F_pendulum[0])):
        for j in range (steps-1):
            if j > start_search:
                if F_pendulum[j,i] < Min_F_pendulum[i]:
                    Min_F_pendulum[i] = round(F_pendulum[j,i])
  
    return (Max_location, Max_acceleration, Max_F_pendulum, Min_F_pendulum,start_search)






def Angle_At_Tide_Calculator(segments, Length_Pendulum, Angle_Pendulum, Angle_Pendulum_Mid_Tide,
                             Pendulum_hinge_top_location, Tidal_level):

    """"
# Function:
# This function calculates the angle of the pendulums when the segments is in rest at the given tidal level

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_Pendulum							= List of lengths of pendulums								(2*segments-2)
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)
# Angle_Pendulum_Mid_Tide					= Angle of pendulums at mid tide							(1)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Tidal_level								= Tidal level to be evaluated								(1)

# Output:
# Angle_Pendulum_Initial_Tidal				= List of angles of pendulum still in water at given tide	(2*segments-2)
# Pendulum_hinge_bottom_location			= Y- and z-coordinate of the bottom hing of pendulum    	(2)
"""

    Pendulum_hinge_bottom_location = np.zeros((2, 2 * segments - 2))

    for i in range(2 * segments - 2):
        Pendulum_hinge_bottom_location[1, i] = Pendulum_hinge_top_location[1] - np.sin(Angle_Pendulum_Mid_Tide) * \
                                               Length_Pendulum[i] - Tidal_level  # m   respect to COG of segment

        z_dif_hinges = Pendulum_hinge_top_location[1] - Pendulum_hinge_bottom_location[1, i]
        Angle_Pendulum[i] = np.arcsin(z_dif_hinges / Length_Pendulum[i])

        Pendulum_hinge_bottom_location[0, i] = Pendulum_hinge_top_location[0] - np.cos(Angle_Pendulum[i]) * \
                                               Length_Pendulum[i]  # m     respect to COG of segment

    Angle_Pendulum_Initial_Tidal = Angle_Pendulum

    return (Angle_Pendulum_Initial_Tidal, Pendulum_hinge_bottom_location)



def Pendulum_Force (iteration_step, x_list, xdot_list, segments, Length_segment, free_space_between_segments, characteristics_small_floater, Angle_Pendulum, Pendulum_hinge_top_location, Stiffness_Pendulum, Damping_Pendulum, F_pendulum, Three_Dimensional_Calculation):

    """"
    NOTHING TO SHOW HERE XXX
    """


    Elongation_pendulums = np.zeros(2*segments-2)
    Movement_pendulums = np.zeros(2*segments-2)

    y_loc_pendulum, z_loc_pendulum = Pendulum_hinge_top_location

    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater

    if Three_Dimensional_Calculation == True:
        MC = 0
    else:
        MC = 7

    for i in range (segments):
        if Three_Dimensional_Calculation == False:
            if i == 0:
                continue
            if i == segments - 1:
                continue

        if i == 0:
            Elongation_pendulums[0] = (x_list[iteration_step,6*i+1-MC] - x_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[0]) + \
                                      (x_list[iteration_step,6*i+2-MC] + x_list[iteration_step,6*i+3-MC] * y_loc_pendulum )  * np.sin(Angle_Pendulum[0])
            Movement_pendulums[0] = (xdot_list[iteration_step, 6*i+1 - MC] - xdot_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[0]) + \
                                    (xdot_list[iteration_step, 6*i+2 - MC] + xdot_list[iteration_step,6*i+3-MC] * y_loc_pendulum ) * np.sin(Angle_Pendulum[0])

        elif i == segments - 1:
            Elongation_pendulums[-1] = (x_list[iteration_step,6*i+1-MC] - x_list[iteration_step,6*i+3-MC] * z_loc_pendulum )* np.cos(Angle_Pendulum[-1]) + \
                                       (x_list[iteration_step,6*i+2-MC] + x_list[iteration_step,6*i+3-MC] * y_loc_pendulum )  * np.sin(Angle_Pendulum[-1])
            Movement_pendulums[-1] = ( xdot_list[iteration_step, 6*i+1 - MC] - xdot_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[-1]) + \
                                     (xdot_list[iteration_step, 6*i+2 - MC] + xdot_list[iteration_step,6*i+3-MC] * y_loc_pendulum )* np.sin(Angle_Pendulum[-1])

        else:
            Elongation_pendulums[2 * i - 1] = (x_list[iteration_step, 6*i+1 - MC] - x_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[2 * i - 1]) + \
                                              (x_list[iteration_step,6*i+ 2 - MC] + x_list[iteration_step,6*i+3-MC] * y_loc_pendulum ) * np.sin(Angle_Pendulum[2 * i - 1])
            Elongation_pendulums[2 * i] = (x_list[iteration_step, 6*i+1 - MC] - x_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[2 * i]) + \
                                          (x_list[iteration_step, 6*i+2 - MC] + x_list[iteration_step,6*i+3-MC] * y_loc_pendulum ) * np.sin(Angle_Pendulum[2 * i])
            Movement_pendulums[2 * i - 1] = (xdot_list[iteration_step, 6*i+1 - MC] - xdot_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[2 * i - 1]) +\
                                            (xdot_list[iteration_step, 6*i+2 - MC] + xdot_list[iteration_step,6*i+3-MC] * y_loc_pendulum )* np.sin(Angle_Pendulum[2 * i - 1])
            Movement_pendulums[2 * i] = (xdot_list[iteration_step, 6*i+1 - MC] - xdot_list[iteration_step,6*i+3-MC] * z_loc_pendulum ) * np.cos(Angle_Pendulum[2 * i]) + \
                                        (xdot_list[iteration_step, 6*i+2 - MC] + xdot_list[iteration_step,6*i+3-MC] * y_loc_pendulum )* np.sin(Angle_Pendulum[2 * i])

    for i in range (len(Elongation_pendulums)):
        F_pendulum[iteration_step+1,i] +=  Elongation_pendulums[i] * Stiffness_Pendulum[i] \
                                           + Movement_pendulums[i] * Damping_Pendulum[i]

    return (F_pendulum)