# import of libraries
import numpy as np



def Tidal_Bridge_M (segments, M_segment,J_segment, Three_Dimensional_Calculation):


    """"
    # Function:
# This function makes the mass matrix for the segments, neglecting all other elements

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# M_segment                         		= Mass of a single segment                              	(1)
# J_segment									= Rotational moment of inertia about x y and z-coordinate   (3)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# M_matrix          						= Mass matrix                                           	(NxN)
    """

    # Split rotational moment of inertia into three directions
    Jx_segment, Jy_segment, Jz_segment = J_segment

    # Size of matrix of complete system, Matrix Reduction factor to allow for storage in 2D matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
        MR = 0  # Scaling for reduced matrix
    else:
        N_matrix = 3
        MR = 1  # Scaling for reduced matrix

    # Create empty matrix to store values in later
    M_matrix = np.zeros((N_matrix,N_matrix))

    # Mass matrix for every row, filter if 2D calculation
    for i in range (N_matrix + MR):

        if Three_Dimensional_Calculation == True:
            if i % 6 == 0:
                M_matrix[i, i] = M_segment #kg

        if i % 6 == 1:
            M_matrix[i- MR, i - MR] = M_segment #kg

        if i % 6 == 2:
            M_matrix[i - MR, i - MR] = M_segment #kg

        if i % 6 == 3:
            M_matrix[i - MR, i - MR] = Jx_segment #kg m2

        if Three_Dimensional_Calculation == True:
            if i % 6 == 4:
                M_matrix[i, i] = Jy_segment #kg m2

            if i % 6 == 5:
                M_matrix[i, i] = Jz_segment #kg m2


    return (M_matrix)







def TB_Force_Water (segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                    characteristics_large_floater, rho, g, Three_Dimensional_Calculation):


    """"
    # Function:
# This function makes the spring matrix for the segments due to the behaviour of water

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravetational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# Matrix									= Temporary matrix which functions as K or C matrix			(NxN)
    """

    # Split floater information
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater

    # Size of matrix of complete system, Matrix Reduction factor to allow for storage in 2D matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
        MR = 0  # Scaling for reduced matrix
    else:
        N_matrix = 3
        MR = 1  # Scaling for reduced matrix

    # Create empty matrix
    Matrix = np.zeros((N_matrix, N_matrix))

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1/2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1/2. * W_large_floater * 0.5 * W_large_floater  # m2

    # Total distance between floaters or pendulum, total length minus small floater 1 meter minus free space
    dist_floaters = Length_segment - W_small_floater - free_space_between_segments  # m

    # Computation of K-matrix for every segment, will later be implemented into complete matrix
    if Three_Dimensional_Calculation == True:
        Matrix_segment = np.zeros((6, 6))
    else:
        Matrix_segment = np.zeros((3, 3))

    # Spring effect of water for vertical
    Matrix_segment[2 - MR, 2 - MR] += rho * g * (N_small_floater * A_small_floater + N_large_floater * A_large_floater)

    # Spring effect of water for rotation about x
    Matrix_segment[3 - MR, 3 - MR] += 1/12. * rho * g * ( N_small_floater * W_small_floater * ( L_small_floater + 0.5
                                            * W_small_floater) ** 3 + N_large_floater * W_large_floater *
                                            ( L_large_floater + 2 * 0.25 * W_large_floater) ** 3)

    if Three_Dimensional_Calculation == True:
        # Spring effect of water for rotation about y. two steps, separated by small and large floaters
        Matrix_segment[4, 4] += 2 * rho * g * A_small_floater * (1 / 2. * dist_floaters) ** 2

        for i in range(int(N_large_floater)):
            Matrix_segment[4, 4] += rho * g * A_large_floater * (
                    dist_floaters / 2 - dist_floaters / (N_large_floater + 1) * (i + 1)) ** 2

    # For 3D calculation, loop for every segment. For 2D, do that not.
    if Three_Dimensional_Calculation == True:
        for i in range(segments):
            for j in range(6):
                for k in range(6):
                    Matrix[i * 6 + j, i * 6 + k] += Matrix_segment[j, k]
    else:
        Matrix = Matrix_segment


    return(Matrix)


def TB_Hydro_Force_Adjacent (Length_segment, free_space_between_segments, characteristics_small_floater,
                    characteristics_large_floater, rho, g,Stiffness_lean_factor):


    """"
    # Function:
# This function determines the hydrodynamic stiffness of the adjacent floating element that the middle two elements experience.

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravetational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# Matrix									= Temporary matrix which functions as K or C matrix			(NxN)
    """

    # Split floater information
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    N_large_floater, W_large_floater, L_large_floater = characteristics_large_floater
    dist_floaters = 19.25

    # Create empty matrix
    Matrix = np.zeros((3, 3))

    # Horizontal area of floaters, including the tips
    A_small_floater = W_small_floater * L_small_floater + 2 * 1/2. * W_small_floater * W_small_floater  # m2
    A_large_floater = W_large_floater * L_large_floater + 2 * 1/2. * W_large_floater * 0.5 * W_large_floater  # m2
    A_total = (A_small_floater * N_small_floater + A_large_floater * N_large_floater)
    
    I_small = (1/12.) * (L_small_floater + W_small_floater) * W_small_floater**3 + (1.5*W_large_floater+0.5*W_small_floater+2*dist_floaters)**2*A_small_floater
    I_large_middle = (1/12.) * (L_large_floater + 0.5 * W_large_floater) * W_large_floater**3
    I_large_side = (1/12.) * (L_large_floater + 0.5 * W_large_floater) * W_large_floater**3 + (W_large_floater+dist_floaters)**2 * A_large_floater
    I_tot = 2*I_small + 2*I_large_side +I_large_middle
    
    ##Normal mass
    F_p = np.linspace(0,0.4 * 2320000 * g, 100) #About 10% of the total weight
    eccentricity = (Length_segment-free_space_between_segments)/2
    M_p = eccentricity*F_p
    
    depth_ini = 0 #Is 0
    F_tot = F_p
    depth_new = F_tot / (A_total * g *rho)
    phi = M_p/(rho*g*I_tot)
    d_phi_h=eccentricity*np.sin(phi)
    dh_tot = abs(depth_ini - (depth_new+d_phi_h))
    
    F_p=F_p[1:]
    dh_tot=dh_tot[1:]
    Matrix[1,1] = np.mean(F_p/dh_tot)*Stiffness_lean_factor *2 #The added hydraulic stiffness comes from the elements on both sides.

    
    return(Matrix)









def TB_Force_Pendulum(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                      Angle_Pendulum, Pendulum_hinge_top_location, kc_pendulum, Three_Dimensional_Calculation):

    """"

# Function:
# This function makes the spring / damper matrix for the segments due to the behaviour of the pendulums

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# kc_pendulum								= Could be both the pendulum stiffness or damping value		(1)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# Matrix									= Temporary matrix which functions as K or C matrix			(NxN)
    """

    # split information
    N_small_floater, W_small_floater, L_small_floater = characteristics_small_floater
    y_loc_pendulum, z_loc_pendulum = Pendulum_hinge_top_location

    # Size of matrix of complete system, Matrix Reduction factor to allow for storage in 2D matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
        MR = 0  #Scaling for reduced matrix
    else:
        N_matrix = 3
        MR = 7  #Scaling for reduced matrix

    # print(kc_pendulum)

    Matrix = np.zeros((N_matrix, N_matrix))

    dist_pendulums = Length_segment - 2 * 0.5 *  W_small_floater - free_space_between_segments  # m

    # fill every element of the matrix, split into pendulum on right side and pendulum on left side.
    # Skips are made for 2D calculation
    for i in range (segments):
        if Three_Dimensional_Calculation == False:
            if i == 0:
                continue
            if i == segments - 1:
                continue

        # All except for last segment, if statements to allow for 2D matrix
        if i != segments - 1 :
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+0-MR, -MR+ i*6+0] += 0
                Matrix[i*6+0-MR, -MR+ i*6+1] += 0
                Matrix[i*6+0-MR, -MR+ i*6+2] += 0
                Matrix[i*6+0-MR, -MR+ i*6+3] += 0
                Matrix[i*6+0-MR, -MR+ i*6+4] += 0
                Matrix[i*6+0-MR, -MR+ i*6+5] += 0



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+1-MR, -MR+ i*6+0] += 0
            Matrix[i*6+1-MR, -MR+ i*6+1] += kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i])
            Matrix[i*6+1-MR, -MR+ i*6+2] += kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i])
            Matrix[i*6+1-MR, -MR+ i*6+3] += y_loc_pendulum * kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                            * np.cos(Angle_Pendulum[2*i]) - z_loc_pendulum * kc_pendulum[2*i] \
                                            * np.cos(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i])
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+1-MR, -MR+ i*6+4] += - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i])
                Matrix[i*6+1-MR, -MR+ i*6+5] += 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i])



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+2-MR, -MR+ i*6+0] += 0
            Matrix[i*6+2-MR, -MR+ i*6+1] += kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i])
            Matrix[i*6+2-MR, -MR+ i*6+2] += kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i])*(1+0)
            Matrix[i*6+2-MR, -MR+ i*6+3] += y_loc_pendulum * kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                            * np.sin(Angle_Pendulum[2*i]) - z_loc_pendulum * kc_pendulum[2*i] \
                                            * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i])
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+2-MR, -MR+ i*6+4] += - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i])
                Matrix[i*6+2-MR, -MR+ i*6+5] += 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i])



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+3-MR, -MR+ i*6+0] += 0
            Matrix[i*6+3-MR, -MR+ i*6+1] += - kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                            * np.cos(Angle_Pendulum[2*i]) * z_loc_pendulum + kc_pendulum[2*i] \
                                            * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) * y_loc_pendulum
            Matrix[i*6+3-MR, -MR+ i*6+2] += - kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                            * np.cos(Angle_Pendulum[2*i]) * z_loc_pendulum + kc_pendulum[2*i] \
                                            * np.sin(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) * y_loc_pendulum
            Matrix[i*6+3-MR, -MR+ i*6+3] += z_loc_pendulum * kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                            * np.cos(Angle_Pendulum[2*i]) * z_loc_pendulum - z_loc_pendulum \
                                            * kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                            * np.sin(Angle_Pendulum[2*i]) * y_loc_pendulum + y_loc_pendulum \
                                            * kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                            * np.sin(Angle_Pendulum[2*i]) * y_loc_pendulum - y_loc_pendulum \
                                            * kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                            * np.cos(Angle_Pendulum[2*i]) * z_loc_pendulum
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+3-MR, -MR+ i*6+4] += - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) \
                                                * y_loc_pendulum + 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i]) \
                                                * z_loc_pendulum
                Matrix[i*6+3-MR, -MR+ i*6+5] += 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) \
                                                * y_loc_pendulum - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i]) \
                                                * z_loc_pendulum



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+4-MR, -MR+ i*6+0] += 0
                Matrix[i*6+4-MR, -MR+ i*6+1] += - kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                                * np.sin(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+2] += - kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                                * np.sin(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+3] += - y_loc_pendulum * kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                                * np.sin(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums \
                                                + z_loc_pendulum * kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                                * np.sin(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+4] += 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) \
                                                * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+5] += - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.sin(Angle_Pendulum[2*i]) \
                                                * 1/2. * dist_pendulums



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+5-MR, -MR+ i*6+0] += 0
                Matrix[i*6+5-MR, -MR+ i*6+1] += kc_pendulum[2*i] * np.cos(Angle_Pendulum[2*i]) \
                                                * np.cos(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+2] += kc_pendulum[2*i] * np.sin(Angle_Pendulum[2*i]) \
                                                * np.cos(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+3] += - z_loc_pendulum * kc_pendulum[2*i]  * np.cos(Angle_Pendulum[2*i]) \
                                                * np.cos(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums \
                                                + y_loc_pendulum * kc_pendulum[2*i]  * np.sin(Angle_Pendulum[2*i]) \
                                                * np.cos(Angle_Pendulum[2*i]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+4] += - 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.sin(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i]) \
                                                * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+5] += 1/2. * dist_pendulums * kc_pendulum[2*i] \
                                                * np.cos(Angle_Pendulum[2*i]) * np.cos(Angle_Pendulum[2*i]) \
                                                * 1/2. * dist_pendulums

        # All except for first segment
        if i != 0:
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+0-MR, -MR+ i*6+0] += 0
                Matrix[i*6+0-MR, -MR+ i*6+1] += 0
                Matrix[i*6+0-MR, -MR+ i*6+2] += 0
                Matrix[i*6+0-MR, -MR+ i*6+3] += 0
                Matrix[i*6+0-MR, -MR+ i*6+4] += 0
                Matrix[i*6+0-MR, -MR+ i*6+5] += 0



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+1-MR, -MR+ i*6+0] += 0
            Matrix[i*6+1-MR, -MR+ i*6+1] += kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1])
            Matrix[i*6+1-MR, -MR+ i*6+2] += kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1])
            Matrix[i*6+1-MR, -MR+ i*6+3] += y_loc_pendulum * kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1]) - z_loc_pendulum * kc_pendulum[2*i-1] \
                                            * np.cos(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1])
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+1-MR, -MR+ i*6+4] += 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1])
                Matrix[i*6+1-MR, -MR+ i*6+5] += - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1])



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+2-MR, -MR+ i*6+0] += 0
            Matrix[i*6+2-MR, -MR+ i*6+1] += kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                            * np.sin(Angle_Pendulum[2*i-1])
            Matrix[i*6+2-MR, -MR+ i*6+2] += kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.sin(Angle_Pendulum[2*i-1])*(1+0)
            Matrix[i*6+2-MR, -MR+ i*6+3] += y_loc_pendulum * kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.sin(Angle_Pendulum[2*i-1]) - z_loc_pendulum * kc_pendulum[2*i-1] \
                                            * np.cos(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1])
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+2-MR, -MR+ i*6+4] += 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1])
                Matrix[i*6+2-MR, -MR+ i*6+5] += - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1])



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+3-MR, -MR+ i*6+0] += 0
            Matrix[i*6+3-MR, -MR+ i*6+1] += - kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1]) * z_loc_pendulum + kc_pendulum[2*i-1] \
                                            * np.cos(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                            * y_loc_pendulum
            Matrix[i*6+3-MR, -MR+ i*6+2] += - kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1]) * z_loc_pendulum + kc_pendulum[2*i-1] \
                                            * np.sin(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                            * y_loc_pendulum
            Matrix[i*6+3-MR, -MR+ i*6+3] += z_loc_pendulum * kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1]) * z_loc_pendulum - z_loc_pendulum \
                                            * kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                            * np.sin(Angle_Pendulum[2*i-1]) * y_loc_pendulum + y_loc_pendulum \
                                            * kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.sin(Angle_Pendulum[2*i-1]) * y_loc_pendulum - y_loc_pendulum \
                                            * kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                            * np.cos(Angle_Pendulum[2*i-1]) * z_loc_pendulum
            if Three_Dimensional_Calculation == True:
                Matrix[i*6+3-MR, -MR+ i*6+4] += 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                                * y_loc_pendulum - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1]) \
                                                * z_loc_pendulum
                Matrix[i*6+3-MR, -MR+ i*6+5] += - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                                * y_loc_pendulum + 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1]) \
                                                * z_loc_pendulum



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+4-MR, -MR+ i*6+0] += 0
                Matrix[i*6+4-MR, -MR+ i*6+1] += kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                                * np.sin(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+2] += kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                                * np.sin(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+3] += y_loc_pendulum * kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                                * np.sin(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums \
                                                - z_loc_pendulum * kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                                * np.sin(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+4] += 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                                * 1/2. * dist_pendulums
                Matrix[i*6+4-MR, -MR+ i*6+5] += - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.sin(Angle_Pendulum[2*i-1]) \
                                                * 1/2. * dist_pendulums



            if Three_Dimensional_Calculation == True:
                Matrix[i*6+5-MR, -MR+ i*6+0] += 0
                Matrix[i*6+5-MR, -MR+ i*6+1] += - kc_pendulum[2*i-1] * np.cos(Angle_Pendulum[2*i-1]) \
                                                * np.cos(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+2] += - kc_pendulum[2*i-1] * np.sin(Angle_Pendulum[2*i-1]) \
                                                * np.cos(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+3] += z_loc_pendulum * kc_pendulum[2*i-1]  * np.cos(Angle_Pendulum[2*i-1]) \
                                                * np.cos(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums \
                                                - y_loc_pendulum * kc_pendulum[2*i-1]  * np.sin(Angle_Pendulum[2*i-1]) \
                                                * np.cos(Angle_Pendulum[2*i-1]) * 1/2. * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+4] += - 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.sin(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1]) \
                                                * 1/2 * dist_pendulums
                Matrix[i*6+5-MR, -MR+ i*6+5] += 1/2. * dist_pendulums * kc_pendulum[2*i-1] \
                                                * np.cos(Angle_Pendulum[2*i-1]) * np.cos(Angle_Pendulum[2*i-1]) \
                                                * 1/2. * dist_pendulums

    return(Matrix)





def TB_Force_Spudpoles(segments, Length_segment, kc_spudpole, Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the spring / damper matrix for the segments due to the behaviour of the spudpoles

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# kc_spudpole								= Could be both the pendulum spudpole or damping value		(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# Matrix									= Temporary matrix which functions as K or C matrix			(NxN)
    """

    # Size of matrix of complete system, Matrix Reduction factor to allow for storage in 2D matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    #Create empty matrix
    Matrix = np.zeros((N_matrix, N_matrix))


    # For 3D calculation, fill matrix with spudpole sping/dampers
    if Three_Dimensional_Calculation == True:
        Matrix[1, 1] += kc_spudpole[0]
        Matrix[1, 5] += - 1 / 2. * Length_segment * kc_spudpole[0]
        Matrix[5, 1] += - 1 / 2. * Length_segment * kc_spudpole[0]
        Matrix[5, 5] += 1 / 2. * Length_segment * kc_spudpole[0] * 1 / 2. * Length_segment

        Matrix[6 * segments - 6 + 1, 6 * segments - 6 + 1] += kc_spudpole[1]
        Matrix[6 * segments - 6 + 1, 6 * segments - 6 + 5] += 1 / 2. * Length_segment * kc_spudpole[1]
        Matrix[6 * segments - 6 + 5, 6 * segments - 6 + 1] += 1 / 2. * Length_segment * kc_spudpole[1]
        Matrix[6 * segments - 6 + 5, 6 * segments - 6 + 5] += 1 / 2. * Length_segment * kc_spudpole[1] * 1 / 2. \
                                                              * Length_segment

    return(Matrix)





def TB_Force_Connection(segments, Length_segment, kc_connection_segments, Three_Dimensional_Calculation):


    """"
    # Function:
# This function makes the spring / damper matrix for the segments due to the behaviour of the connecting elements

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# kc_connection_segments					= Could be both the connection stiffness or damping value 	(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# Matrix									= Temporary matrix which functions as K or C matrix			(NxN)
"""

    # split information
    kc_y_connection_segments, kc_z_connection_segments = kc_connection_segments

    # Size of matrix of complete system, Matrix Reduction factor to allow for storage in 2D matrix
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Create empty matrix
    Matrix = np.zeros((N_matrix, N_matrix))

    # for 3D calculation, create spring/damper force between the various elements
    if Three_Dimensional_Calculation == True:
        for i in range(segments):

            # For every segment that is not the last one
            if i != segments - 1:
                # Effect of translation / rotation on itself
                Matrix[6 * i + 1, 6 * i + 1] += kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 2] += kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 4] += 1 / 2. * Length_segment * kc_y_connection_segments * 1 / 2. \
                                                * Length_segment
                Matrix[6 * i + 5, 6 * i + 5] += 1 / 2. * Length_segment * kc_z_connection_segments * 1 / 2. \
                                                * Length_segment

                # Effect of translation / rotation on it, interaction elements
                Matrix[6 * i + 1, 6 * i + 5] += kc_y_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 2, 6 * i + 4] += -kc_z_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 4, 6 * i + 2] += -kc_y_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 5, 6 * i + 1] += kc_z_connection_segments * 1 / 2. * Length_segment

                # Effect of translation / rotation on neighbour + interaction
                Matrix[6 * i + 1, 6 * i + 1 + 6] += -kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 2 + 6] += -kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 4 + 6] += 1 / 2. * Length_segment * kc_y_connection_segments * 1 / 2. \
                                                    * Length_segment
                Matrix[6 * i + 5, 6 * i + 5 + 6] += 1 / 2. * Length_segment * kc_z_connection_segments * 1 / 2. \
                                                    * Length_segment

                Matrix[6 * i + 1, 6 * i + 5 + 6] += 1 / 2. * Length_segment * kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 4 + 6] += - 1 / 2. * Length_segment * kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 2 + 6] += 1 / 2. * Length_segment * kc_y_connection_segments
                Matrix[6 * i + 5, 6 * i + 1 + 6] += - 1 / 2. * Length_segment * kc_z_connection_segments

            # For every segment that is not the first one
            if i != 0:
                # Effect of translation / rotation on it
                Matrix[6 * i + 1, 6 * i + 1] += kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 2] += kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 4] += 1 / 2. * Length_segment * kc_y_connection_segments * 1 / 2. \
                                                * Length_segment
                Matrix[6 * i + 5, 6 * i + 5] += 1 / 2. * Length_segment * kc_z_connection_segments * 1 / 2. \
                                                * Length_segment

                # Effect of translation / rotation on it, interaction elements
                Matrix[6 * i + 1, 6 * i + 5] += -kc_y_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 2, 6 * i + 4] += kc_z_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 4, 6 * i + 2] += kc_y_connection_segments * 1 / 2. * Length_segment
                Matrix[6 * i + 5, 6 * i + 1] += -kc_z_connection_segments * 1 / 2. * Length_segment

                # Effect of translation / rotation on neighbour + interaction
                Matrix[6 * i + 1, 6 * i + 1 - 6] += -kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 2 - 6] += -kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 4 - 6] += 1 / 2. * Length_segment * kc_y_connection_segments * 1 / 2. \
                                                    * Length_segment
                Matrix[6 * i + 5, 6 * i + 5 - 6] += 1 / 2. * Length_segment * kc_z_connection_segments * 1 / 2. \
                                                    * Length_segment

                Matrix[6 * i + 1, 6 * i + 5 - 6] += - 1 / 2. * Length_segment * kc_y_connection_segments
                Matrix[6 * i + 2, 6 * i + 4 - 6] += 1 / 2. * Length_segment * kc_z_connection_segments
                Matrix[6 * i + 4, 6 * i + 2 - 6] += - 1 / 2. * Length_segment * kc_y_connection_segments
                Matrix[6 * i + 5, 6 * i + 1 - 6] += 1 / 2. * Length_segment * kc_z_connection_segments

    return(Matrix)





def TB_Spring_Matrix_constant(segments, Length_segment, Stiffness_Spudpole, Stiffness_connection_segment, rho, g,
                              Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the spring matrix for all constant (over time) terms

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# characteristics_large_floater     		= Number of, width and length of large floaters         	(3)
# Stiffness_Spudpole						= List of stiffnesses of the two spudpoles					(2)
# Stiffness_connection_segment				= Stiffness of connecting elements							(1)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravetational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# K_matrix          						= Spring matrix                                         	(NxN)
    """

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Empty matrix
    K_Matrix = np.zeros((N_matrix, N_matrix))

    # Various elements to store in complete matrix
    K_Spudpole = TB_Force_Spudpoles(segments, Length_segment, Stiffness_Spudpole, Three_Dimensional_Calculation)
    K_Connection = TB_Force_Connection(segments, Length_segment, Stiffness_connection_segment,
                                       Three_Dimensional_Calculation)

    K_Matrix += K_Spudpole + K_Connection

    return(K_Matrix)





def TB_Damping_Matrix_constant(segments, Length_segment, Damping_Spudpole, Damping_connection_segment,
                               Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the damping matrix for all constant (over time) terms

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# Damping_Spudpole							= List of dampers of the two spudpoles						(2)
# Damping_connection_segment				= Dampers of connecting elements							(1)
# rho                               		= Density of the fluid                                  	(1)
# g											= Gravitational acceleration								(1)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# C_matrix          						= Damper matrix                                         	(NxN)
    """

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Empty matrix
    C_Matrix = np.zeros((N_matrix, N_matrix))

    # Various elements to store in complete matrix
    C_Spudpole = TB_Force_Spudpoles(segments, Length_segment, Damping_Spudpole, Three_Dimensional_Calculation)
    C_Connection = TB_Force_Connection(segments, Length_segment, Damping_connection_segment,
                                       Three_Dimensional_Calculation)

    C_Matrix += C_Spudpole + C_Connection

    return (C_Matrix)





def TB_Spring_Matrix_variable(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                              Angle_Pendulum, Pendulum_hinge_top_location, Stiffness_Pendulum,
                              Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the spring matrix for all non-constant (over time) terms

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Stiffness_Pendulum						= List of stiffnesses of pendulums							(2*segments-2)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# K_matrix          						= Spring matrix                                         	(NxN)
    """

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Empty matrix
    K_Matrix = np.zeros((N_matrix, N_matrix))

    # Various elements to store in complete matrix
    K_Pendulum = TB_Force_Pendulum(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                                   Angle_Pendulum, Pendulum_hinge_top_location, Stiffness_Pendulum,
                                   Three_Dimensional_Calculation)

    K_Matrix += K_Pendulum

    return(K_Matrix)






def TB_Damping_Matrix_variable(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                               Angle_Pendulum, Pendulum_hinge_top_location, Damping_Pendulum,
                               Three_Dimensional_Calculation):

    """"
    # Function:
# This function makes the spring matrix for all non-constant (over time) terms

# Input:
# segments                          		= Number of segments in the bridge                      	(1)
# Length_segment							= Length of a single segment								(1)
# free_space_between_segments				= Free space between the small floaters between segments	(1)
# characteristics_small_floater     		= Number of, width and length of small floaters         	(3)
# Angle_Pendulum							= List of the angles of all pendulums						(2*segments-2)
# Pendulum_hinge_top_location				= Y- and z-coordinate of the top hing of pendulum       	(2)
# Damping_Pendulum							= List of dampers of pendulums								(2*segments-2)
# Three_Dimensional_Calculation				= Option to implement three dimensional calculation			(T/F)

# Output:
# C_matrix          						= Damper matrix                                         	(NxN)
    """

    # Size of matrix of complete system
    if Three_Dimensional_Calculation == True:
        N_matrix = 6 * segments
    else:
        N_matrix = 3

    # Empty matrix
    C_Matrix = np.zeros((N_matrix, N_matrix))

    # Various elements to store in complete matrix
    C_Pendulum = TB_Force_Pendulum(segments, Length_segment, free_space_between_segments, characteristics_small_floater,
                                   Angle_Pendulum, Pendulum_hinge_top_location, Damping_Pendulum,
                                   Three_Dimensional_Calculation,Added_heave_stiffness_factor)

    C_Matrix += C_Pendulum

    return (C_Matrix)
