import matplotlib.pyplot as plt
import numpy as np
#import pandas as pd
from numpy import genfromtxt
from sympy import tanh, sinh, sqrt, symbols, nsolve
from pylab import figure, show, legend, ylabel

#Winddata is found at https://app.metoceanview.com/hindcast/sites/indo/-8/123.2
#The used coordinate was Indonesia 8S 123.2E

data = genfromtxt('data.csv', delimiter=',',usecols=(1,2,3,4,5,6,7,8),skip_header=1)

Umin=0      #Min wind speed in knots for figures and calculations
Umax=40   #Max wind speed in knots for figures
UImax = 40 #Max winds peed in knots for probability calculations
step=500 #Amount of steps within the Weibull curves
binwidth=5 #Amount of knots per bin
g=9.8   
depth=23    #Average depth in the relevant area around the Tidal Bridge
floater_length = 34
draught = 2.1



#%%

#Data processing for fitting Weibull curves through the wind data
count = 0
for i in range(len(data[:,1])):
    for j in range(len(data[1,:])):
        count += data[i,j]

xbindata=np.linspace(Umin+2.5,Umax-2.5,len(data[1,:])) #x-values for histogram
N=data[0]/(count) #y-values for histogram
NE=data[1]/(count) #y-values for histogram
E=data[2]/(count) #y-values for histogram
SE=data[3]/(count) #y-values for histogram
S=data[4]/(count) #y-values for histogram
SW=data[5]/(count) #y-values for histogram
W=data[6]/(count) #y-values for histogram
NW=data[7]/(count) #y-values for histogram

Ntot,NEtot,Etot,SEtot,Stot,SWtot,Wtot,NWtot=0,0,0,0,0,0,0,0

for i in range(len(N)):
    Ntot+=N[i]
    NEtot+=NE[i]
    Etot+=E[i]
    SEtot+=SE[i]
    Stot+=S[i]
    SWtot+=SW[i]
    Wtot+=W[i]
    NWtot+=NW[i]

#Function to plot the weibull curves
def weibull(Umax, step,k,a,fac,title,data):
    U=np.linspace(0,Umax,step)
    dU=U[1]-U[0]
    probU = np.zeros(len(U))
    for i in range(step):
        probU[i] = ((k/a)*(U[i]/a)**(k-1)*np.exp(-(U[i]/a)**k))*fac
#    plt.figure()
#    plt.bar(xbindata,data/binwidth)
#    plt.plot(U,probU)
#    plt.xlabel('Wind speed (knots)')
#    plt.ylabel('Probability of occurance per knot (-)')
#    plt.title('Wind direction '+title+' has a total probability of occurance of '+str(np.round(fac,decimals=4)))
#    plt.legend(('Weibull fit: k = '+str(k)+', a = '+str(a), 'Wind data','test'))
#    plt.savefig('Prob_Of_Wave_Length_of_direction_'+str(title)+'.png',dpi=300)
    return(probU,U,dU,fac)

#Plot the curves
Nprob = weibull(Umax,step, 2.,6,Ntot,'N',N)
NEprob = weibull(Umax,step, 2.05,7.45,NEtot,'NE',NE)
Eprob = weibull(Umax,step, 2.1,7.9,Etot,'E',E)
SEprob = weibull(Umax,step, 2.2,7.3,SEtot,'SE',SE)
Sprob = weibull(Umax,step, 2.0,7.7,Stot,'S',S)
SWprob = weibull(Umax,step, 2.05,7.4,SWtot,'SW',SW)
Wprob = weibull(Umax,step, 1.85,10.8,Wtot,'W',W)
NWprob = weibull(Umax,step, 2.1,10.,NWtot,'NW',NW)

#%%


def integrate(UImin,UImax,Probvalues,Uvalues,dU):
    #Function to find the probability of occurance
    summation=0
    for i in range(len(Probvalues)):
        if Uvalues[i]>UImin and Uvalues[i]<UImax:
            summation+=Probvalues[i]*dU
    return(summation)

#totprob = 0            
#totprob += integrate(UImin,UImax,Nprob[0],Nprob[1],Nprob[2])
#totprob += integrate(UImin,UImax,NEprob[0],NEprob[1],NEprob[2])
#totprob += integrate(UImin,UImax,Eprob[0],Eprob[1],Eprob[2])
#totprob += integrate(UImin,UImax,SEprob[0],SEprob[1],SEprob[2])
#totprob += integrate(UImin,UImax,Sprob[0],Sprob[1],Sprob[2])
#totprob += integrate(UImin,UImax,SWprob[0],SWprob[1],SWprob[2])
#totprob += integrate(UImin,UImax,Wprob[0],Wprob[1],Wprob[2])
#totprob += integrate(UImin,UImax,NWprob[0],NWprob[1],NWprob[2])



#Formula's
def Hsolver(Hslim,dF,F):
    #Function to calculate the needed wind speed in knots for a certain wave height, depth and fetch combination.
    U10m = symbols('U10m')
    func = (U10m**2/g)*0.24*(tanh(0.343*(g*dF/U10m**2)**1.14)*tanh(4.41*10**(-4)*(g*F/U10m**2)**0.79/tanh(0.343*(g*dF/U10m**2)**1.14)))**0.572 - Hslim
    U10m = nsolve(func,10,verify=False) #U10 in m/s
    U10k = U10m*3.6/1.85166
    Tp = U10m/g*7.69*(tanh(0.1*(g*dF/U10m**2)**2.01)*tanh(2.77*10**(-7)*(g*F/U10m**2)**1.45/tanh(0.1*(g*dF/U10m**2)**2.01)))**0.187
    Lp0 = 80
    Lp1 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*dF/Lp0)
    Lp2 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*dF/Lp1)
    Lp3 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*dF/Lp2)
    Lp4 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*dF/Lp3)
    k = 2*np.pi/Lp4
    c = g*Tp/(2*np.pi)*tanh(2*np.pi*dF/Lp3)
    cg = 0.5*c*(1+2*k*dF/sinh(2*k*dF))
    return(U10k,Tp,Lp4,c,cg)

def WaveData(Tp,depth=23,current=0):
    #Function to calculate the wave data
    Lp0 = 80
    Lp1 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*depth/Lp0)
    Lp2 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*depth/Lp1)
    Lp3 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*depth/Lp2)
    Lp4 = g*Tp**2/(2*np.pi)*tanh(2*np.pi*depth/Lp3)
    k = 2*np.pi/Lp4
    c = g*Tp/(2*np.pi)*tanh(2*np.pi*depth/Lp3)
    cg = 0.5*c*(1+2*k*depth/sinh(2*k*depth))
    kn = symbols('kn')
    func = sqrt(g*kn*tanh(kn*depth))+kn*current-2*np.pi/Tp
    wnc = nsolve(func,0,verify=False)
    Lnc = 2*np.pi/wnc
    sigma = sqrt(g*wnc*tanh(wnc*depth))
    currentfac = sigma/(2*np.pi/Tp)
    return(Lp4,k,c,cg,currentfac,Lnc)
#%%

Wavedat1 = Hsolver(1,2000,660000) #Calculate wave period
Wavedat2 = WaveData(Wavedat1[1],18) #
kp_s = sqrt(Wavedat2[3]/Wavedat1[4])#shoaling
print(kp_s,1.0039/1.0057, 1.0143/1.00396)

#%%




def probabilitycalculator(Hpos_loc,N=4):
    #Calculating the probability for a given wave height at the location of the Tidal Bridge
        
        #N=4: The 
        Hneg_loc = Hpos_loc
        
        #Find the near shore processes coefficients for positive traveling waves (Waves traveling from North-East to South-West)
        #Shoaling
        Wavedat1 = Hsolver(Hpos_loc,2000,660000) #Calculate wave period
        Wavedat2 = WaveData(Wavedat1[1],depth) #
        kp_s = sqrt(Wavedat2[3]/Wavedat1[4])#shoaling
        kp_s_s = kp_s*0.998
        kp_s_l = kp_s*1.01
#        kp_s_s = kp_s
#        kp_s_l = kp_s
        #Current
        kp_c = 1.35
        kp_c_s = 1.15
        kp_c_l = 1.5 #Determined with the mails of reiniers and the eurlier written python script.
        kp_c = 1.35
        kp_c_s = 1.35
        kp_c_l = 1.35 #Determined with the mails of reiniers and the eurlier written python script.
        #Near shore refraction
#        kp_f = 0.85 # funneling/Refraction
#        kp_f_s = 0.71 # funneling/Refraction
#        kp_f_l = 1 # funneling/Refraction
        kp_f = 0.85 # funneling/Refraction
        kp_f_s = 0.85 # funneling/Refraction
        kp_f_l = 0.85 # funneling/Refraction
        #Dissipation
        kp_dis = 1/(1+Hpos_loc**2*0.02)#dissipation guess
        kp_dis_s = kp_dis
        kp_dis_l = kp_dis
#        kp_dis_s = 1/(1+Hpos_loc**2*0.04)#dissipation
#        kp_dis_l = 1/(1+Hpos_loc**2*0.0)#dissipation
        #Refraction
        kp_ref_N = np.cos(30/180*np.pi)
#        kp_ref_N_s = 0.98*kp_ref_N
#        kp_ref_N_l = 1.02*kp_ref_N
        kp_ref_N_s = kp_ref_N
        kp_ref_N_l = kp_ref_N
        kp_ref_NE = np.cos(10/180*np.pi)
#        kp_ref_NE_s = 0.98* kp_ref_NE
#        kp_ref_NE_l = 1.02* kp_ref_NE
        kp_ref_NE_s = kp_ref_NE
        kp_ref_NE_l = kp_ref_NE
        kp_ref_E = 1. #No refraction but strong diffraction
        kp_ref_E_s = 1. #No refraction but strong diffraction
        kp_ref_E_l = 1. #No refraction but strong diffraction
        kp_ref_W = 1. #No refraction but strong diffraction
        kp_ref_W_s = 1. #No refraction but strong diffraction
        kp_ref_W_l = 1. #No refraction but strong diffraction
        kp_ref_NW = 1.0
        kp_ref_NW_s = 1.
        kp_ref_NW_l = 1.
        #Diffraction
        kp_dif_N = 1.
        kp_dif_N_s = 1.
        kp_dif_N_l = 1.
        kp_dif_NE = 1.
        kp_dif_NE_s = 1.
        kp_dif_NE_l = 1.
        kp_dif_E = 0.5 #Waves need to make a turn but go strait THIS ONE
        kp_dif_E_s = 0.1 #Waves need to make a turn but go strait
        kp_dif_E_l = 0.7 #Waves need to make a turn but go strait
        kp_dif_E_s = 0.5 #Waves need to make a turn but go strait
        kp_dif_E_l = 0.5 #Waves need to make a turn but go strait
        kp_dif_W = 0.02 #Waves need to make a turn but go strait
        kp_dif_W_s = 0.01 #Waves need to make a turn but go strait
        kp_dif_W_l = 0.04 #Waves need to make a turn but go strait
        kp_dif_W_s = 0.02 #Waves need to make a turn but go strait
        kp_dif_W_l = 0.02 #Waves need to make a turn but go strait
        kp_dif_NW = 0.25#Waves need to make a turn but go strait THIS ONE VERY MUCH
        kp_dif_NW_s = 0.1 #Waves need to make a turn but go strait
        kp_dif_NW_l = 0.4 #Waves need to make a turn but go strait
        kp_dif_NW_s = 0.25 #Waves need to make a turn but go strait
        kp_dif_NW_l = 0.25 #Waves need to make a turn but go strait
        Hpos_sea_N = Hpos_loc / (kp_s * kp_c * kp_f * kp_dis * kp_ref_N * kp_dif_N)
        Hpos_sea_N_s = Hpos_loc / (kp_s_s * kp_c_s * kp_f_s * kp_dis_s * kp_ref_N_s * kp_dif_N_s)
        Hpos_sea_N_l = Hpos_loc / (kp_s_l * kp_c_l * kp_f_l * kp_dis_l * kp_ref_N_l * kp_dif_N_l)
        Hpos_sea_NE = Hpos_loc / (kp_s_l * kp_c * kp_f * kp_dis * kp_ref_NE * kp_dif_NE)
        Hpos_sea_NE_s = Hpos_loc / (kp_s_s * kp_c_s * kp_f_s * kp_dis_s * kp_ref_NE_s * kp_dif_NE_s)
        Hpos_sea_NE_l = Hpos_loc / (kp_s_l * kp_c_l * kp_f_l * kp_dis_l * kp_ref_NE_l * kp_dif_NE_l)
        Hpos_sea_E = Hpos_loc / (kp_s * kp_c * kp_f * kp_dis * kp_ref_E * kp_dif_E)
        Hpos_sea_E_s = Hpos_loc / (kp_s_s * kp_c_s * kp_f_s * kp_dis_s * kp_ref_E_s * kp_dif_E_s)
        Hpos_sea_E_l = Hpos_loc / (kp_s_l * kp_c_l * kp_f_l * kp_dis_l * kp_ref_E_l * kp_dif_E_l)
        Hpos_sea_W = Hpos_loc / (kp_s * kp_c * kp_f * kp_dis * kp_ref_W * kp_dif_W)
        Hpos_sea_W_s = Hpos_loc / (kp_s_s * kp_c_s * kp_f_s * kp_dis_s * kp_ref_W_s * kp_dif_W_s)
        Hpos_sea_W_l = Hpos_loc / (kp_s_l * kp_c_l * kp_f_l * kp_dis_l * kp_ref_W_l * kp_dif_W_l)
        Hpos_sea_NW = Hpos_loc / (kp_s * kp_c * kp_f * kp_dis * kp_ref_NW * kp_dif_NW)
        Hpos_sea_NW_s = Hpos_loc / (kp_s_s * kp_c_s * kp_f_s * kp_dis_s * kp_ref_NW_s * kp_dif_NW_s)
        Hpos_sea_NW_l = Hpos_loc / (kp_s_l * kp_c_l * kp_f_l * kp_dis_l * kp_ref_NW_l * kp_dif_NW_l)
#        print(Hpos_sea_N,Hpos_sea_NE,Hpos_sea_E,Hpos_sea_W,Hpos_sea_NW)
        
        #Find the near shore processes coefficients for negative traveling waves (Waves traveling from South-West to North-East)
        #Shoaling
        Wavedat1 = Hsolver(Hneg_loc,200,25000)
        Wavedat2 = WaveData(Wavedat1[1],depth)
        kn_s = sqrt(Wavedat2[3]/Wavedat1[4])
        kn_s_s = kn_s*0.998
        kn_s_l = kn_s*1.01
        kn_c = kp_c #current
        kn_c_s = kp_c_s
        kn_c_l = kp_c_l
        kn_f = kp_f # funneling
        kn_f_s = kp_f_s # funneling
        kn_f_l = kp_f_l # funneling
        kn_dis = kp_dis #dissipation
        kn_dis_s = kp_dis_s #dissipation
        kn_dis_l = kp_dis_l #dissipation
        kn_ref_SE = 1. #No refraction
        kn_ref_SE_s = 1. #No refraction
        kn_ref_SE_l = 1. #No refraction
        kn_ref_S = 0.85
        kn_ref_S_s = 0.8
        kn_ref_S_l = 0.9
        kn_ref_S_s = 0.85
        kn_ref_S_l = 0.85
        kn_ref_SW = 0.99
        kn_ref_SW_s = 0.98
        kn_ref_SW_l = 1.00
        kn_ref_SW_s = 0.99
        kn_ref_SW_l = 0.99
        
        kn_dif_SE = 0.25 #diffraction
        kn_dif_SE_s = 0.05 #diffraction
        kn_dif_SE_l = 0.4 #diffraction
        kn_dif_SE_s = 0.25 #diffraction
        kn_dif_SE_l = 0.25 #diffraction
        kn_dif_S = 1. #diffraction
        kn_dif_S_s = 1. #diffraction
        kn_dif_S_l = 1. #diffraction
        kn_dif_SW = 1. #diffraction
        kn_dif_SW_s = 1. #diffraction
        kn_dif_SW_l = 1. #diffraction
        Hneg_sea_SE = Hneg_loc / (kn_s * kn_c * kn_f * kn_dis * kn_ref_SE * kn_dif_SE)
        Hneg_sea_SE_s = Hneg_loc / (kn_s_s * kn_c_s * kn_f_s * kn_dis_s * kn_ref_SE_s * kn_dif_SE_s)
        Hneg_sea_SE_l = Hneg_loc / (kn_s_l * kn_c_l * kn_f_l * kn_dis_l * kn_ref_SE_l * kn_dif_SE_l)
        Hneg_sea_S = Hneg_loc / (kn_s * kn_c * kn_f * kn_dis * kn_ref_S * kn_dif_S)
        Hneg_sea_S_s = Hneg_loc / (kn_s_s * kn_c_s * kn_f_s * kn_dis_s * kn_ref_S_s * kn_dif_S_s)
        Hneg_sea_S_l = Hneg_loc / (kn_s_l * kn_c_l * kn_f_l * kn_dis_l * kn_ref_S_l * kn_dif_S_l)
        Hneg_sea_SW = Hneg_loc / (kn_s * kn_c * kn_f * kn_dis * kn_ref_SW * kn_dif_SW)
        Hneg_sea_SW_s = Hneg_loc / (kn_s_s * kn_c_s * kn_f_s * kn_dis_s * kn_ref_SW_s * kn_dif_SW_s)
        Hneg_sea_SW_l = Hneg_loc / (kn_s_l * kn_c_l * kn_f_l * kn_dis_l * kn_ref_SW_l * kn_dif_SW_l)
#        print(Hneg_sea_S, Hneg_sea_S)
        
        #Calculate the wind speeds in knots and the wave periods
        UImin_N = Hsolver(Hpos_sea_N,2000,270000)
        UImin_N_s = Hsolver(Hpos_sea_N_s,2000,270000)
        UImin_N_l = Hsolver(Hpos_sea_N_l,2000,270000)
        UImin_NE = Hsolver(Hpos_sea_NE,2000,600000)
        UImin_NE_s = Hsolver(Hpos_sea_NE_s,2000,600000)
        UImin_NE_l = Hsolver(Hpos_sea_NE_l,2000,600000)
        UImin_E = Hsolver(Hpos_sea_E,2000,300000)
        UImin_E_s = Hsolver(Hpos_sea_E_s,2000,300000)
        UImin_E_l = Hsolver(Hpos_sea_E_l,2000,300000)
        UImin_SE = Hsolver(Hneg_sea_SE,200,8000)
        UImin_SE_s = Hsolver(Hneg_sea_SE_s,200,8000)
        UImin_SE_l = Hsolver(Hneg_sea_SE_l,200,8000)
        UImin_S = Hsolver(Hneg_sea_S,200,6000)
        UImin_S_s = Hsolver(Hneg_sea_S_s,200,6000)
        UImin_S_l = Hsolver(Hneg_sea_S_l,200,6000)
        UImin_SW = Hsolver(Hneg_sea_SW,200,25000)
        UImin_SW_s = Hsolver(Hneg_sea_SW_s,200,25000)
        UImin_SW_l = Hsolver(Hneg_sea_SW_l,200,25000)
        UImin_W = Hsolver(Hpos_sea_W,200,500000)
        UImin_W_s = Hsolver(Hpos_sea_W_s,200,500000)
        UImin_W_l = Hsolver(Hpos_sea_W_l,200,500000)
        UImin_NW = Hsolver(Hpos_sea_NW,200,450000)
        UImin_NW_s = Hsolver(Hpos_sea_NW_s,200,450000)
        UImin_NW_l = Hsolver(Hpos_sea_NW_l,200,450000)
        
        Tps = np.array([UImin_N[1],UImin_NE[1],UImin_E[1],UImin_SE[1],UImin_S[1],UImin_SW[1],UImin_W[1],UImin_NW[1]]).astype(np.object)
        Tps_s = np.array([UImin_N_s[1],UImin_NE_s[1],UImin_E_s[1],UImin_SE_s[1],UImin_S_s[1],UImin_SW_s[1],UImin_W_s[1],UImin_NW_s[1]]).astype(np.object)
        Tps_l = np.array([UImin_N_l[1],UImin_NE_l[1],UImin_E_l[1],UImin_SE_l[1],UImin_S_l[1],UImin_SW_l[1],UImin_W_l[1],UImin_NW_l[1]]).astype(np.object)
#        Lps = np.array([UImin_N[2],UImin_NE[2],UImin_E[2],UImin_SE[2],UImin_S[2],UImin_SW[2],UImin_W[2],UImin_NW[2]]).astype(np.object)
        
        #Probability = integration of (min wind speed in knots, max wind speed, probability y values, correstponding x/U values, dU)
        probN = integrate(UImin_N[0],UImax,Nprob[0],Nprob[1],Nprob[2])
        probN_s = integrate(UImin_N_s[0],UImax,Nprob[0],Nprob[1],Nprob[2])
        probN_l = integrate(UImin_N_l[0],UImax,Nprob[0],Nprob[1],Nprob[2])
        probNE = integrate(UImin_NE[0],UImax,NEprob[0],NEprob[1],NEprob[2])
        probNE_s = integrate(UImin_NE_s[0],UImax,NEprob[0],NEprob[1],NEprob[2])
        probNE_l = integrate(UImin_NE_l[0],UImax,NEprob[0],NEprob[1],NEprob[2])
        probE = integrate(UImin_E[0],UImax,Eprob[0],Eprob[1],Eprob[2])
        probE_s = integrate(UImin_E_s[0],UImax,Eprob[0],Eprob[1],Eprob[2])
        probE_l = integrate(UImin_E_l[0],UImax,Eprob[0],Eprob[1],Eprob[2])
        probSE = integrate(UImin_SE[0],UImax,SEprob[0],SEprob[1],SEprob[2])
        probSE_s = integrate(UImin_SE_s[0],UImax,SEprob[0],SEprob[1],SEprob[2])
        probSE_l = integrate(UImin_SE_l[0],UImax,SEprob[0],SEprob[1],SEprob[2])
        probS = integrate(UImin_S[0],UImax,Sprob[0],Sprob[1],Sprob[2])
        probS_s = integrate(UImin_S_s[0],UImax,Sprob[0],Sprob[1],Sprob[2])
        probS_l = integrate(UImin_S_l[0],UImax,Sprob[0],Sprob[1],Sprob[2])
        probSW = integrate(UImin_SW[0],UImax,SWprob[0],SWprob[1],SWprob[2])
        probSW_s = integrate(UImin_SW_s[0],UImax,SWprob[0],SWprob[1],SWprob[2])
        probSW_l = integrate(UImin_SW_l[0],UImax,SWprob[0],SWprob[1],SWprob[2])
        probW = integrate(UImin_W[0],UImax,Wprob[0],Wprob[1],Wprob[2])
        probW_s = integrate(UImin_W_s[0],UImax,Wprob[0],Wprob[1],Wprob[2])
        probW_l = integrate(UImin_W_l[0],UImax,Wprob[0],Wprob[1],Wprob[2])
        probNW = integrate(UImin_NW[0],UImax,NWprob[0],NWprob[1],NWprob[2])
        probNW_s = integrate(UImin_NW_s[0],UImax,NWprob[0],NWprob[1],NWprob[2])
        probNW_l = integrate(UImin_NW_l[0],UImax,NWprob[0],NWprob[1],NWprob[2])
        prob = np.array([probN,probNE,probE,probSE,probS,probSW,probW,probNW])
        prob_s = np.array([probN_s,probNE_s,probE_s,probSE_s,probS_s,probSW_s,probW_s,probNW_s])
        prob_l = np.array([probN_l,probNE_l,probE_l,probSE_l,probS_l,probSW_l,probW_l,probNW_l])
        totprob = probN+probNE+probE+probSE+probS+probSW+probW+probNW
        totprob_s = probN_s+probNE_s+probE_s+probSE_s+probS_s+probSW_s+probW_s+probNW_s
        totprob_l = probN_l+probNE_l+probE_l+probSE_l+probS_l+probSW_l+probW_l+probNW_l
        
#        prob = np.around(prob.astype(np.double),5)
        Tps = np.around(Tps.astype(np.double),1)
        Tps_s = np.around(Tps_s.astype(np.double),1)
        Tps_l = np.around(Tps_l.astype(np.double),1)
#        Lps = np.around(Lps.astype(np.double),1)
        probtemp=np.zeros(len(prob))
        probtemp_s=np.zeros(len(prob))
        probtemp_l=np.zeros(len(prob))
        for i in range(len(prob)):
            probtemp[i]=prob[i]
            probtemp_s[i]=prob_s[i]
            probtemp_l[i]=prob_l[i]
        probnew = np.zeros(len(Tps))
        probnew_s = np.zeros(len(Tps_s))
        probnew_l = np.zeros(len(Tps_l))
        for j in range(N):
            for i in range(len(Tps)):
                if probtemp[i] == max(probtemp):
                    probnew[i] = probtemp[i]
                    probtemp[i]=0
                    break
        for j in range(N):
            for i in range(len(Tps_s)):
                if probtemp_s[i] == max(probtemp_s):
                    probnew_s[i] = probtemp_s[i]
                    probtemp_s[i]=0
                    break
        for j in range(N):
            for i in range(len(Tps_l)):
                if probtemp_l[i] == max(probtemp_l):
                    probnew_l[i] = probtemp_l[i]
                    probtemp_l[i]=0
                    break
        
        Lps=np.zeros(len(prob))
        Lps_s=np.zeros(len(prob_s))
        Lps_l=np.zeros(len(prob_l))
        Lps_avg = 0
        Lps_avg_s = 0
        Lps_avg_l = 0
        Tps_avg = 0
        Tps_avg_s = 0
        Tps_avg_l = 0
        for i in range(len(Lps)):
            Lps[i],a,b,c,d,e=WaveData(Tps[i])
            Lps_s[i],a_s,b_s,c_s,d_s,e_s=WaveData(Tps_s[i])
            Lps_l[i],a_l,b_l,c_l,d_l,e_l=WaveData(Tps_l[i])
            Lps_avg += Lps[i]*prob[i]/totprob
            Lps_avg_s += Lps_s[i]*prob_s[i]/totprob_s
            Lps_avg_l += Lps_l[i]*prob_l[i]/totprob_l
            Tps_avg+= Tps[i]*prob[i]/totprob
            Tps_avg_s+= Tps_s[i]*prob_s[i]/totprob_s
            Tps_avg_l+= Tps_l[i]*prob_l[i]/totprob_l
        for i in range(len(Tps)):
            if probnew[i] == 0:
                Tps[i]=0
                Lps[i]=0
            if probnew_s[i] == 0:
                Tps_s[i]=0
                Lps_s[i]=0
            if probnew_l[i] == 0:
                Tps_l[i]=0
                Lps_l[i]=0

        totprob_all = [totprob,totprob_s,totprob_l]
        prob_all = [prob,prob_s,prob_l]
        Tps_all = [Tps,Tps_s,Tps_l]
        Lps_all = [Lps,Lps_s, Lps_l]
        Tps_avg_all = [Tps_avg,Tps_avg_s,Tps_avg_l]
        Lps_avg_all = [Lps_avg, Lps_avg_s, Lps_avg_l]
        #totprob: Total probability of occurance for given wave height
        #prob: Probability of occurance of given wave height per direction
        #Tps: Significant wave period for give wave height per direction
        #Lps: Significant wave length for give wave height per direction
        #Lps_avg: weighted average of the significant wave length for give wave height per direction
        return(totprob_all,prob_all,Tps_all,Lps_all,Tps_avg_all,Lps_avg_all)



#%%


def WaveProbRel(Hmax,step=25):
    #Function that plots figures:
    #   -Probability of exceedance of a given wave height
    #   -Wave height and wave length relation
    #   -Dimensionless: Wave height and wave length relation
    
    #Hmax: The maximum wave height taken into account for the plots
    array = np.zeros(step)
    for i in range(step):
        y=i**3
        array[i]=y
    Hpos_loc = array*Hmax/max(array)
    totprob = np.zeros((len(Hpos_loc),3))
    Lps_avg = np.zeros((len(Hpos_loc),3))
    Tps_avg = np.zeros((len(Hpos_loc),3))
    for i in range(step):
        totprob[i],a,b,c,Tps_avg[i],Lps_avg[i] = probabilitycalculator(Hpos_loc[i])
#    print('Probability of exeedance is P =',np.round(totprob,decimals=4))
#    print('Hpos_loc',Hpos_loc)
    omega=np.zeros((len(Hpos_loc),3))
    for i in range(len(Hpos_loc)):
        omega[i] = 2*np.pi/Tps_avg[i]
    
    
    np.savetxt("Wave_height_probability.csv",Hpos_loc, delimiter=',')
    np.savetxt("Wave_period_probability.csv",Tps_avg, delimiter=',')
    
    
    plt.figure()
    plt.plot(Tps_avg,totprob)
    plt.title('Probability of exceedance')
    plt.xlabel('Significant wave period')
    plt.ylabel('Probability of exceedance (-)')
    plt.grid()
    plt.savefig('Prob_Of_Exceedance_vs_Wave_Period.png',dpi=300)
    
    plt.figure()
    plt.plot(Hpos_loc,totprob)
    plt.title('Probability of exceedance')
    plt.xlabel('Significant wave height at Tidal Bridge location (m)')
    plt.ylabel('Probability of exceedance (-)')
    plt.grid()
    plt.savefig('Prob_Of_Exceedance.png',dpi=300)
    
    plt.figure()
    plt.plot(Hpos_loc,Lps_avg)
    plt.title('Most probable wave length per wave height')
    plt.xlabel('Significant wave height at Tidal Bridge location (m)')
    plt.ylabel('Most probable wave length (m)')
    plt.grid()
    plt.savefig('Prob_Of_Wave_Length.png',dpi=300)
    plt.figure()
    plt.plot(Hpos_loc/draught,Lps_avg/floater_length)
    plt.title('Most probable wave length per wave height')
    plt.xlabel('Dimensionless wave heigth H/d(-)')
    plt.ylabel('Dimensionless wave length Lw/Lf (-)')
    plt.xlim((0.25,1.25))
    plt.ylim((0.5,4))
    plt.grid()
    plt.savefig('Prob_Of_Dimensionless_Wave_Length.png',dpi=300)
    plt.figure()
    plt.plot(Hpos_loc,Tps_avg)
    plt.title('Wave period per wave height')
    plt.xlabel('Significant wave height')
    plt.ylabel('Significant wave period')
#    plt.xlim((0.25,1.25))
#    plt.ylim((0.5,4))
    plt.grid()
    plt.savefig('Wave_Period_Wave_Length.png',dpi=300)
    plt.figure()
    plt.plot(Tps_avg,Hpos_loc)
    plt.title('Most propable wave height per wave period')
    plt.xlabel('Significant wave period')
    plt.ylabel('Significant wave height')
#    plt.xlim((0.25,1.25))
#    plt.ylim((0.5,4))
    plt.grid()
    plt.savefig('Wave_Height_vs_Wave_Period.png',dpi=300)
    
    fig1 = figure()
    ax1 = fig1.add_subplot(111)
    plt.xlim((0.55, 1.65))
    line1 = ax1.plot(omega, Hpos_loc, 'purple')
    ax1.yaxis.tick_right()
    ax1.yaxis.set_label_position("right")
    ylabel("Wave height [m]")
#    legend((line1),('1'))
    plt.savefig('Wave_Period_Wave_Length2.png',dpi=300)
    
    
WaveProbRel(3.54,10)


 #%%


#Fill in the wave height to find the probability of occurance and the corresponding wave period and wave length

#Wave_test_height = 1.187 #1.187 gives a prop of 5 days downtime
Wave_test_height = 1.5 #Significant wave height in meters
Wave = probabilitycalculator(Wave_test_height,4)
print('Exceeded',np.round(Wave[0][0]*365,1),' days per year with in a range of',
      np.round(Wave[0][1]*365,1),'and', np.round(Wave[0][2]*365,1))
print('Probability of exceedance',np.round(Wave[0][0],5))
print('Probability of exceedance',np.round(Wave[0][0]*365,2))
print('Significant wave period',np.round(Wave[4][0],2),' within a range of ',np.round(Wave[4][1],2) ,' and ',np.round(Wave[4][2],2))
print('Significant angular frequency',np.round(2*np.pi/Wave[4][0],2))
print('Significant wave length',np.round(Wave[5][0],2))


print(Wave[3])
print(Wave[4])
#totprob_all,prob_all,Tps_all,Lps_all,Tps_avg_all,Lps_avg_all